package com.yutou.nas_music_player.Adapters;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.RecyclerView;

import com.alibaba.fastjson.JSONObject;
import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.AbsObjectInterface;
import com.yutou.nas_music_player.Interfaces.DefaultNetworkInterface;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.R;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.network.HttpManager;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.AppTools;
import com.yutou.nas_music_player.tools.ImageLoader;
import com.yutou.nas_music_player.tools.StringUtil;
import com.yutou.nas_music_player.views.AlbumsActivity;

import java.util.ArrayList;
import java.util.List;

import cc.ibooker.ztextviewlib.MarqueeTextView;
import jp.wasabeef.glide.transformations.RoundedCornersTransformation;

public class MusicLibsAdapter extends RecyclerView.Adapter<MusicLibsAdapter.ViewHolder> {
    public static final int LIBS_MODEL_ALL = 0;
    public static final int LIBS_MODEL_Album = 1;
    public static final int LIBS_MODEL_Artist = 2;
    public static final int LIBS_MODEL_Tmp = 3;
    public static final int LIBS_MODEL_PLAY_LIST = 4;
    public static final int LIBS_MODEL_PLAY_Collection = 5;

    public interface OnClickAction {
        void onClick(Intent intent, View view);
    }

    protected OnClickAction clickAction;

    public void setOnClickAction(OnClickAction onClickAction) {
        this.clickAction = onClickAction;
    }


    List<MusicData> list = new ArrayList<>();
    Context context;
    MediaBrowserHelper browserHelper;

    public MusicLibsAdapter(Context context) {
        this.context = context;
    }

    private int model = -1;

    private String musicPath = "root";

    public void initData(int model, boolean init) {
        this.model = model;
        //list.clear();
        System.out.println("填充序列 设置model " + model + " " + this.model + " &");

    }

    @Override
    public int getItemViewType(int position) {
        return model;
    }


    public void initData(int model) {
        initData(model, false);
    }


    private int scrollStatus = 0;

    public void setScrollStatus(int scrollStatus) {
        this.scrollStatus = scrollStatus;
    }


    public void onDestroy() {
    }

    public void setData(List<MusicData> list) {
        this.list = list;
    }

    public void setPath(String path) {
        this.musicPath = path;
    }

    public int getModel() {
        return model;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.layout_music_libs_item, parent, false);
        switch (viewType) {
            case LIBS_MODEL_Album:
                return new ViewHolderAlbum(view);
            case LIBS_MODEL_PLAY_Collection:
                return new ViewHolderCollection(view);
            default:
                return new ViewHolderAll(view);
        }
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.bind(list.get(position));
        System.out.println("list.size() = " + list.size());
    }

    @Override
    public int getItemCount() {
        return list.size();
    }

    public MusicData getItem(int position) {
        return list.get(position);
    }

    public void setBrowserHelper(MediaBrowserHelper browserHelper) {
        this.browserHelper = browserHelper;
    }

    public void setModel(int model) {
        this.model = model;
    }

    public class ViewHolderCollection extends ViewHolder {

        ViewHolderCollection(View itemView) {
            super(itemView);
        }

        @Override
        public void bind(MusicData data) {
            super.bind(data);
            collection.setVisibility(View.GONE);
            message.setText("共 " + data.getComment() + " 首歌");
            message.setVisibility(View.VISIBLE);
            artist.setVisibility(View.VISIBLE);
            collectionDir(data);
            topDir(data);
            itemView.setOnClickListener(v -> clickCollection(data));
            icon.setImageResource(R.drawable.ic_dir);
        }
    }

    public class ViewHolderAll extends ViewHolder {

        ViewHolderAll(View itemView) {
            super(itemView);
        }

        @Override
        public void bind(MusicData data) {
            super.bind(data);
            collectionDir(data);
            topDir(data);
            itemView.setOnClickListener(v -> {
                        clickAllList(data);
                    }
            );
            itemView.setOnLongClickListener(new View.OnLongClickListener() {
                @Override
                public boolean onLongClick(View view) {
                    showPOPWindows(data);
                    return true;
                }
            });
        }
    }

    public class ViewHolderAlbum extends ViewHolder {

        ViewHolderAlbum(View itemView) {
            super(itemView);
        }

        @Override
        public void bind(MusicData data) {
            super.bind(data);
            message.setVisibility(View.GONE);
            collectionAlbum(data);
            topAlbum(data);
            itemView.setOnClickListener(v -> clickAlbum(data));
        }


    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        MarqueeTextView title, artist, message;
        LinearLayout buttons;
        ImageButton collection, top, dislike;
        ImageView icon, play_mask;

        ViewHolder(View itemView) {
            super(itemView);
            title = itemView.findViewById(R.id.title);
            artist = itemView.findViewById(R.id.artist);
            message = itemView.findViewById(R.id.message);
            buttons = itemView.findViewById(R.id.buttons);
            collection = itemView.findViewById(R.id.collection);
            top = itemView.findViewById(R.id.top);
            dislike = itemView.findViewById(R.id.dislike);
            icon = itemView.findViewById(R.id.album_image);
            play_mask = itemView.findViewById(R.id.play_mask);

            icon.setTag("icon");
            title.setTag("title");
            artist.setTag("artist");
            message.setTag("message");
            buttons.setTag("buttons");
            collection.setTag("collection");
            top.setTag("top");
            dislike.setTag("dislike");
            play_mask.setTag("play_mask");

        }

        protected void collectionDir(MusicData data) {
            collection.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (data.isDir()) {
                        JSONObject json = new JSONObject();
                        json.put("path", data.getFile());
                        json.put("type", true);
                        MusicContainer.getLibs().onDirMusicData(json, new NetworkInterface() {
                            @Override
                            public void httpGetData(Object data1, int state) {
                                List<MusicData> list = (List<MusicData>) data1;
                                MusicContainer.getLibs().onAddCollectionDir(itemView.getContext(), list.toArray(new MusicData[0]));
                            }

                            @Override
                            public void httpError(Exception e) {

                            }
                        });
                    } else {
                        MusicContainer.getLibs().onAddCollectionDir(itemView.getContext(), data);
                    }

                }
            });
        }

        protected void collectionAlbum(MusicData data) {
            collection.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (data.isDir()) {
                        MusicContainer.getLibs().onAlbumMusicData(data.getAlbum(), new NetworkInterface() {
                            @Override
                            public void httpGetData(Object md, int state) {
                                List<MusicData> list = (List<MusicData>) md;
                                MusicContainer.getLibs().onAddCollectionDir(itemView.getContext(), list.toArray(new MusicData[0]));
                            }

                            @Override
                            public void httpError(Exception e) {

                            }
                        });
                    } else {
                        MusicContainer.getLibs().onAddCollectionDir(itemView.getContext(), data);
                    }

                }
            });

        }

        protected void topAlbum(MusicData data) {
            top.setOnClickListener(v -> addAlbumList(data.getTitle()));
        }

        public void topDir(MusicData data) {
            top.setOnClickListener(v -> {
                if (data.isDir()) {
                    showAddTmpListDialog(data);
                } else {
                    MusicContainer.getInstance().addTmpList(data);
                    Toast.makeText(v.getContext(), "已添加到队列中", Toast.LENGTH_LONG).show();
                    for (MusicData musicData : MusicContainer.getInstance().getTmpPlayList()) {
                        System.out.println("临时队列:" + musicData.getTitle());
                    }
                }
            });
        }

        protected void clickCollection(MusicData data) {
            if (data == null)
                return;
            if (StringUtil.isEmpty(data.getComment())) {
                Toast.makeText(itemView.getContext(), "未收藏任何歌曲", Toast.LENGTH_LONG).show();
                return;
            }
            Intent intent = new Intent(itemView.getContext(), AlbumsActivity.class);
            intent.putExtra("model", 2);
            intent.putExtra("fid", data.getId());
            intent.putExtra("image", HttpManager.NetworkAPI.HOME+data.getImageUrl());
            if (clickAction != null) {
                clickAction.onClick(intent, icon);
            }

        }

        protected void clickAlbum(MusicData data) {
            System.out.println("点击item");
            if (data == null)
                return;
            ImageLoader.getInstance(context).getImageBitmap(data.getImageUrl(), AppData.imageWidth, AppData.imageHigh, new AbsObjectInterface<Bitmap>() {
                @Override
                public void onData(Bitmap bitmap) {
                    if(bitmap!=null){
                        Intent intent = new Intent(itemView.getContext(), AlbumsActivity.class);
                        intent.putExtra("model", 0);
                        intent.putExtra("album", data.getAlbum());
                        intent.putExtra("image",  data.getImageUrl());
                        intent.putExtra("color", AppTools.getImageColor(bitmap));
                        if (clickAction != null) {
                            clickAction.onClick(intent, icon);
                        }
                    }
                }
            });

        }


        protected void clickAllList(MusicData data) {
            if (data.isDir()) {
                changeList(data.getFile());
            } else {
                if (browserHelper != null) {
                    browserHelper.play(data);
                    notifyDataSetChanged();
                } else {
                    System.out.println("browserHelper为空");
                }
            }
        }

        protected void changeList(String path) {
            AppTools.showHud(context);
            MusicContainer.getLibs().onDirMusicData(path, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    List<MusicData> list = (List<MusicData>) data;
                    musicPath = path;
                    AppTools.hideHud();
                    setData(list);
                    notifyDataSetChanged();
                }

                @Override
                public void httpError(Exception e) {
                    AppTools.hideHud();
                }
            });
        }


        public void bind(MusicData data) {
            title.setText(data.getTitle());
            artist.setText(data.getArtist());
            //icon.setImageResource(R.drawable.ic_dir);
            collection.setVisibility(View.VISIBLE);
            if (!data.isDir()) {
                message.setText(data.getBitRate() + "kbps | " + data.getSampleRate() + "hz | " + data.getEncodingType());
             /*   message.setVisibility(View.GONE);
                artist.setVisibility(View.GONE);*/
                //   icon.setImageBitmap(ThumbnailUtils.extractThumbnail(data.getImg(), 64,64));
                showImage(data, this);

            } else {
                message.setText("");
                play_mask.setVisibility(View.GONE);
                icon.setImageResource(R.drawable.ic_dir);
            }
            if (MusicContainer.getInstance().getNowPlayData().getMd5().equals(data.getMd5())) {
                play_mask.setVisibility(View.VISIBLE);
            } else {
                play_mask.setVisibility(View.GONE);
            }
        }

        protected void addAlbumList(String album) {
            System.out.println("添加专辑:" + album);
            MusicContainer.getLibs().onAlbumMusicData(album, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    List<MusicData> list = (List<MusicData>) data;
                    for (MusicData musicData : list) {
                        System.out.println(musicData.getTitle() + "| 专辑名 =" + musicData.getAlbum());
                        MusicContainer.getInstance().addTmpList(musicData);
                    }
                    Toast.makeText(itemView.getContext(), "已添加到队列中", Toast.LENGTH_LONG).show();
                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }

        protected void showAddTmpListDialog(MusicData data) {
            AlertDialog dialog = new AlertDialog.Builder(itemView.getContext())
                    .setTitle("添加文件夹到队列")
                    .setMessage("将文件夹中所有歌曲添加到队列?")
                    .setPositiveButton("添加", new DialogInterface.OnClickListener() {

                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            AppTools.showHud(itemView.getContext());
                            JSONObject json = new JSONObject();
                            json.put("path", data.getFile());
                            json.put("type", true);
                            MusicContainer.getLibs().onDirMusicData(json, new NetworkInterface() {
                                @Override
                                public void httpGetData(Object data, int state) {
                                    List<MusicData> list = (List<MusicData>) data;
                                    for (MusicData musicData : list) {
                                        MusicContainer.getInstance().addTmpList(musicData);
                                    }
                                    AppTools.hideHud();
                                    AppTools.toast(list.size() + "首歌已添加到队列中");

                                }

                                @Override
                                public void httpError(Exception e) {
                                    AppTools.toast("添加失败:" + e.getMessage());
                                }
                            });
                        }
                    })
                    .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                            dialog.dismiss();
                        }
                    }).create();
            dialog.show();
        }


        protected void showImage(MusicData musicData, ViewHolder holder) {
            ImageLoader.getInstance(context).showImage(holder.icon,musicData.getImageUrl(),AppData.minImageWidth, AppData.minImageHigh,true);
        }

        PopupWindow popupWindow = null;

        protected void showPOPWindows(MusicData data) {
            Button button = new Button(context);
            button.setText("添加到列表");
            button.setOnClickListener(view -> {
                AppTools.showHud(context);
                if(!data.isDir()){
                    MusicContainer.getInstance().addPlayList(data);
                    AppTools.hideHud();
                    AppTools.toast( "已添加到队列中");
                    popupWindow.dismiss();
                    return;
                }
                JSONObject json = new JSONObject();
                json.put("path", data.getFile());
                json.put("type", true);
                MusicContainer.getLibs().onDirMusicData(json, new NetworkInterface() {
                    @Override
                    public void httpGetData(Object data1, int state) {
                        List<MusicData> list = (List<MusicData>) data1;
                        int i = 0;
                        for (MusicData musicData : list) {
                            if (!musicData.isDir()) {
                                MusicContainer.getInstance().addPlayList(musicData);
                                i++;
                            }
                        }
                        AppTools.hideHud();
                        AppTools.toast(i + "首歌已添加到队列中");
                        popupWindow.dismiss();
                        MusicContainer.getInstance().setPlayModel(MusicContainer.PLAY_MODEL_ORDER);
                        browserHelper.getTransportControls().play();
                        browserHelper.getTransportControls().skipToNext();
                    }

                    @Override
                    public void httpError(Exception e) {
                        AppTools.toast("添加失败:" + e.getMessage());
                    }
                });
            });
            popupWindow = new PopupWindow(button, ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            popupWindow.setOutsideTouchable(true);

            popupWindow.showAsDropDown(itemView, 200, 0);
        }
    }
}
