package com.yutou.nas_music_player.Datas;


import android.graphics.Bitmap;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.room.ColumnInfo;
import androidx.room.Entity;
import androidx.room.Ignore;
import androidx.room.PrimaryKey;

import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.network.HttpManager;
import com.yutou.nas_music_player.tools.AppTools;
import com.yutou.nas_music_player.tools.StringUtil;

@Entity
public class MusicData {
    @PrimaryKey(autoGenerate = true)
    private int id;
    @ColumnInfo(name = "artist")
    private String artist;//艺术家
    @ColumnInfo(name = "album")
    private String album;//专辑
    @ColumnInfo(name = "title")
    private String title;//标题
    @ColumnInfo(name = "comment")
    private String comment;//评论
    @ColumnInfo(name = "year")
    private String year;//年份
    @ColumnInfo(name = "track")
    private String track;//音轨号
    @ColumnInfo(name = "disc_no")
    private String disc_no;//碟片编号
    @ColumnInfo(name = "composer")
    private String composer;//作曲
    @ColumnInfo(name = "artist_sort")
    private String artist_sort;//分类
    @ColumnInfo(name = "lastDir")
    private String lastDir;//上一个文件夹
    @ColumnInfo(name = "file")
    private String file;
    @ColumnInfo(name = "isDir")
    private boolean isDir = false;

    @Ignore
    private boolean isDownloadImg = false;

    @ColumnInfo
    private int bitRate;//比特率
    @ColumnInfo
    private int sampleRate;//采样率
    @ColumnInfo
    private long noOfSamples;//采样数
    @ColumnInfo
    private int channelCount;//声道
    @ColumnInfo
    private String encodingType;//解码类型
    @ColumnInfo
    private double durationAsDouble;//持续时长
    @ColumnInfo
    private boolean lossless;//无损
    @ColumnInfo
    private boolean variableBitRate;//固定码率
    @ColumnInfo
    private String md5;//确保是同一个文件

    @Ignore
    private Bitmap bitmap;

    public MusicData() {
    }


    public void setDownloadImg(boolean downloadImg) {
        isDownloadImg = downloadImg;
    }

    public String getArtist() {
        return artist;
    }

    public void setArtist(String artist) {
        this.artist = artist;
    }

    public String getAlbum() {
        return album;
    }

    public void setAlbum(String album) {
        this.album = album;
    }

    public String getTitle() {
        if (StringUtil.isEmpty(title)) {
            title = album;
        }
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getYear() {
        if (StringUtil.isEmpty(year)) {
            year = "8888";
        }
        return year;
    }

    public void setYear(String year) {
        this.year = year;
    }

    public String getTrack() {
        if (StringUtil.isEmpty(track)) {
            return "0";
        }
        try {
            Integer.parseInt(track);
        } catch (Exception e) {
            return track.substring(0, 1);
        }
        return track;
    }

    public void setTrack(String track) {
        this.track = track;
    }

    public String getDisc_no() {
        if (StringUtil.isEmpty(disc_no)) {
            return "0";
        }
        try {
            Integer.parseInt(disc_no);
        } catch (Exception e) {
            return disc_no.substring(0, 1);
        }
        return disc_no;
    }

    public void setDisc_no(String disc_no) {
        this.disc_no = disc_no;
    }

    public String getComposer() {
        return composer;
    }

    public void setComposer(String composer) {
        this.composer = composer;
    }

    public String getArtist_sort() {
        return artist_sort;
    }

    public void setArtist_sort(String artist_sort) {
        this.artist_sort = artist_sort;
    }


    public String getLastDir() {
        return lastDir;
    }

    public void setLastDir(String lastDir) {
        this.lastDir = lastDir;
    }

    public boolean isDir() {
        return isDir;
    }

    public void setDir(boolean dir) {
        isDir = dir;
    }

    public String getFile() {
        return file;
    }

    public void setFile(String file) {
        this.file = file;
    }

    public String getFileBase64() {
        if (StringUtil.isEmpty(file))
            return null;
        return AppTools.getUrlBase64(file);
    }


    public String getPlayUrl() {
        String url =HttpManager.NetworkAPI.MUSIC_PLAY
                + "?random=false&token=" + HttpManager.NetworkAPI.HTTP_KEY + "&filePath=" + getMd5();
        System.out.println("播放地址:" + url);
        return url;
    }


    public String getImageUrl() {
        return HttpManager.NetworkAPI.MUSIC_IMAGE + "?fileName=" + getMd5();
    }



    public int getBitRate() {
        return bitRate;
    }

    public void setBitRate(int bitRate) {
        this.bitRate = bitRate;
    }

    public int getSampleRate() {
        return sampleRate;
    }

    public void setSampleRate(int sampleRate) {
        this.sampleRate = sampleRate;
    }

    public long getNoOfSamples() {
        return noOfSamples;
    }

    public void setNoOfSamples(long noOfSamples) {
        this.noOfSamples = noOfSamples;
    }

    public int getChannelCount() {
        return channelCount;
    }

    public void setChannelCount(int channelCount) {
        this.channelCount = channelCount;
    }

    public String getEncodingType() {
        return encodingType;
    }

    public void setEncodingType(String encodingType) {
        this.encodingType = encodingType;
    }

    public double getDurationAsDouble() {
        return durationAsDouble;
    }

    public void setDurationAsDouble(double durationAsDouble) {
        this.durationAsDouble = durationAsDouble;
    }

    public boolean isLossless() {
        return lossless;
    }

    public void setLossless(boolean lossless) {
        this.lossless = lossless;
    }

    public boolean isVariableBitRate() {
        return variableBitRate;
    }

    public void setVariableBitRate(boolean variableBitRate) {
        this.variableBitRate = variableBitRate;
    }

    public String getMd5() {
        return md5;
    }

    public void setMd5(String md5) {
        this.md5 = md5;
    }

    @Override
    public boolean equals(@Nullable Object obj) {
        if (obj instanceof MusicData) {
            MusicData data = (MusicData) obj;
            if (StringUtil.isEmpty(md5) || StringUtil.isEmpty(data.getMd5())) {
                return false;
            }
            return md5.equals(data.getMd5());
        }
        return false;
    }

    @Override
    public String toString() {
        return "MusicData{" +
                "id=" + id +
                ", artist='" + artist + '\'' +
                ", album='" + album + '\'' +
                ", title='" + title + '\'' +
                ", comment='" + comment + '\'' +
                ", year='" + year + '\'' +
                ", track='" + track + '\'' +
                ", disc_no='" + disc_no + '\'' +
                ", composer='" + composer + '\'' +
                ", artist_sort='" + artist_sort + '\'' +
                ", lastDir='" + lastDir + '\'' +
                ", file='" + file + '\'' +
                ", isDir=" + isDir +
                ", bitRate=" + bitRate +
                ", sampleRate=" + sampleRate +
                ", noOfSamples=" + noOfSamples +
                ", channelCount=" + channelCount +
                ", encodingType='" + encodingType + '\'' +
                ", durationAsDouble=" + durationAsDouble +
                ", lossless=" + lossless +
                ", variableBitRate=" + variableBitRate +
                ", md5='" + md5 + '\'' +
                '}';
    }

    @NonNull
    @Override
    public MusicData clone() {
        MusicData data = new MusicData();
        data.setId(id);
        data.setArtist(artist);
        data.setAlbum(album);
        data.setTitle(title);
        data.setComment(comment);
        data.setYear(year);
        data.setTrack(track);
        data.setDisc_no(disc_no);
        data.setComposer(composer);
        data.setArtist_sort(artist_sort);
        data.setLastDir(lastDir);
        data.setFile(file);
        data.setDir(isDir);
        data.setBitRate(bitRate);
        data.setSampleRate(sampleRate);
        data.setNoOfSamples(noOfSamples);
        data.setChannelCount(channelCount);
        data.setEncodingType(encodingType);
        data.setDurationAsDouble(durationAsDouble);
        data.setLossless(lossless);
        data.setVariableBitRate(variableBitRate);
        data.setMd5(md5);
        return data;
    }
}
