package com.yutou.nas_music_player;

import android.app.Activity;
import android.app.Application;
import android.content.res.AssetManager;
import android.os.Build;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.yutou.nas_music_player.tools.ImageLoader;
import com.yutou.nas_music_player.views.OpenActivity;

import java.lang.ref.WeakReference;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.Arrays;

import dalvik.system.BaseDexClassLoader;
import dalvik.system.DexClassLoader;
import dalvik.system.DexFile;


public class MyApplication extends Application {
    public static Application application;
    public static DexClassLoader loader;
    private static WeakReference<Activity> activityWeakReference = null;

    public static Activity getActivity() {
        return activityWeakReference.get();
    }

    @Override
    public void onCreate() {
        super.onCreate();
        MyApplication.application = this;
        ImageLoader.getInstance(this);
        registerActivityLifecycleCallbacks(new ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(@NonNull Activity activity, @Nullable Bundle savedInstanceState) {
                activityWeakReference = new WeakReference<>(activity);
            }

            @Override
            public void onActivityStarted(@NonNull Activity activity) {

            }

            @Override
            public void onActivityResumed(@NonNull Activity activity) {

            }

            @Override
            public void onActivityPaused(@NonNull Activity activity) {

            }

            @Override
            public void onActivityStopped(@NonNull Activity activity) {

            }

            @Override
            public void onActivitySaveInstanceState(@NonNull Activity activity, @NonNull Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(@NonNull Activity activity) {

            }
        });
      //  dexLoader();
        System.err.println("加载版本111:" + OpenActivity.version);
    }

    private void dexLoader() {
        File path = new File("/data/data/" + getPackageName() + "/plugs");
        if (!path.exists()) {
            path.mkdirs();
        }
        File jar = new File("/data/data/" + getPackageName() + "/plugs/classes.jar");
        System.err.println("jar = " + jar.getAbsolutePath());
        if (!jar.exists()) {
            System.out.println("进这里了");
            saveAdJar("classes.jar");
            return;
        }
        try {
            Object[] t1 = getDexElements(getClassLoader()); //获取系统类加载器的dexElements数组
            loader = new DexClassLoader(jar.getAbsolutePath(), "/data/data/" + getPackageName() + "/plugs/", null, getClassLoader());
            Object version = loader.loadClass("com.yutou.nas_music_player.views.OpenActivity").getField("version").get(null);
            System.out.println("loader version = " + version);
            Object[] t2 = getDexElements(loader);  //获取jar包类加载器的dexElements数组
            saveLibPath(t1, t2); //合并两个数组，并将合并后的数组覆盖到系统类加载器
            removeLib(loader); //删掉jar包加载器的数组
            System.out.println("加载完成");
            getDexElements(getClassLoader());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 合并两个Elements数组并覆盖掉系统加载类的dexElements
     *
     * @param t1 合成的Elements
     * @param t2 被合并的Elements
     * @throws Exception 未知异常
     */
    private void saveLibPath(Object[] t1, Object[] t2) throws Exception {
        //遍历BaseDexClassLoader，拿到pathList变量 源码参考： 46行
        //遍历BaseDexClassLoader源码：https://android.googlesource.com/platform/libcore/+/refs/heads/master/dalvik/src/main/java/dalvik/system/BaseDexClassLoader.java
        //DexPathList类的源码：       https://android.googlesource.com/platform/libcore/+/refs/heads/master/dalvik/src/main/java/dalvik/system/DexPathList.java
        Field[] fields = BaseDexClassLoader.class.getDeclaredFields();
        Field pathListField = null;
        for (Field field : fields) {
            System.out.println("field.getName() = " + field.getName());
            if (field.getName().equals("pathList")) {
                pathListField = field;
            }
        }
        if (pathListField == null)
            return;
        pathListField.setAccessible(true);
        Object pathList = pathListField.get(getClassLoader());
        Field dexElements = pathList.getClass().getDeclaredField("dexElements"); //获取到系统类加载器的dexElements
        dexElements.setAccessible(true);
        //合并两个获取到系统类加载器的dexElements
        Object[] obj = Arrays.copyOf(t1, t1.length + t2.length);
        int index = 0;
      /*  for (int i = 0; i < t2.length; i++) {
            obj[t1.length+i]=obj[index];
            obj[index++]=t2[i];
        }*/
       /* for (int i=t1.length,j=0;i<t1.length+t2.length;i++,j++){
            Array.set(obj,i,t2[j]);

        }*/
        System.out.println("obj.length = " + obj.length);
        System.arraycopy(t2, 0, obj, t1.length, t2.length);
        Object tmp = obj[0];
        obj[0] = obj[3];
        obj[3] = tmp;
        //覆盖操作
        dexElements.set(pathList, obj);
    }

    /**
     * 移除类加载器的dexElements，不然会提示jar包有多个类加载器
     *
     * @param loader 被移除dexElements的加载器
     */
    private void removeLib(ClassLoader loader) {
        try {
            Field[] fields = BaseDexClassLoader.class.getDeclaredFields();
            Field pathListField = null;
            for (Field field : fields) {
                if (field.getName().equals("pathList"))
                    pathListField = field;
            }
            if (pathListField == null)
                return;
            pathListField.setAccessible(true);
            Object pathList = pathListField.get(loader);
            Field dexElements = pathList.getClass().getDeclaredField("dexElements");
            dexElements.setAccessible(true);
            dexElements.set(pathList, null);//设为null后就不要再使用这个加载器了，否则会报空指针
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取类加载器的DexElements数组
     *
     * @param loader 类加载器
     * @return dexElements
     */
    private Object[] getDexElements(ClassLoader loader) {
        System.out.println("===============start============");
        try {
            Field[] fields = BaseDexClassLoader.class.getDeclaredFields();
            Field pathListField = null;
            for (Field field : fields) {
                System.out.println(field.getName());
                if (field.getName().equals("pathList"))
                    pathListField = field;
            }
            if (pathListField == null)
                return null;
            pathListField.setAccessible(true);
            Object pathList = pathListField.get(loader);
            Field dexElementsField = pathList.getClass().getDeclaredField("dexElements");
            dexElementsField.setAccessible(true);
            Object[] dexElements = (Object[]) dexElementsField.get(pathList);
            //for仅仅只是用于打印Elements类信息的
            String pathStr = "path";
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.O)
                pathStr = "dir";
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M)
                pathStr = "file";
            for (Object lib : dexElements) {
                System.out.println("lib = " + lib);
                Field uri_F = lib.getClass().getDeclaredField(pathStr);
                Field localName_F = lib.getClass().getDeclaredField("dexFile");
                uri_F.setAccessible(true);
                localName_F.setAccessible(true);
                File path = (File) uri_F.get(lib);
                DexFile dexFile = (DexFile) localName_F.get(lib);
                //System.out.println("uri = " + path.getAbsolutePath()+" localName = "+(dexFile==null));
            }
            System.out.println("dexElements length = " + dexElements.length);
            System.out.println("===============end============");
            return dexElements;
        } catch (Exception e) {
            e.printStackTrace();
        }
        System.out.println("===============end error============");
        return null;
    }

    /**
     * 从assets中释放jar包资源
     *
     * @param jarName
     */
    private void saveAdJar(String jarName) {
        AssetManager manager = this.getResources().getAssets();
        try {
            InputStream inputStream = manager.open(jarName);
            FileOutputStream outputStream = new FileOutputStream("/data/data/" + this.getPackageName() + "/plugs/" + jarName);
            if (outputStream == null) {
                return;

            }
            int len = 0;
            byte[] bytes = new byte[inputStream.available()];
            while ((len = inputStream.read(bytes)) != -1) {
                outputStream.write(bytes, 0, len);
            }
            dexLoader();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
