package com.yutou.nas_music_player.containers;

import android.content.ComponentName;
import android.content.Context;
import android.os.RemoteException;
import android.support.v4.media.MediaBrowserCompat;
import android.support.v4.media.MediaMetadataCompat;
import android.support.v4.media.session.MediaControllerCompat;
import android.support.v4.media.session.MediaSessionCompat;
import android.support.v4.media.session.PlaybackStateCompat;

import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.AbsObjectInterface;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.services.MusicService;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.views.PlayLibsActivity;

import java.util.ArrayList;
import java.util.List;

public class MediaBrowserHelper {
    private static MediaBrowserHelper helper;
    private MediaControllerCompat controllerCompat;
    private MediaBrowserCompat mMediaBrowser;
    private final MediaBrowserConnectionCallback browserConnectionCallback;
    private final MediaControllerCallback controllerCallback;
    private final MediaBrowserSubscriptionCallback subscriptionCallback;
    private final Context context;

    private MediaBrowserHelper(Context context) {
        this.context = context;
        browserConnectionCallback = new MediaBrowserConnectionCallback();
        controllerCallback = new MediaControllerCallback();
        subscriptionCallback = new MediaBrowserSubscriptionCallback();
        mMediaBrowser = new MediaBrowserCompat(context, new ComponentName(context, MusicService.class), browserConnectionCallback, null);
        mMediaBrowser.connect();
        MusicContainer.getInstance().addBrowserHelper(this);
    }
    public static MediaBrowserHelper getInstance(){
        if(helper==null){
            helper=new MediaBrowserHelper(MyApplication.application);
        }
        return helper;
    }

    public void onStop() {
        if (controllerCompat != null) {
            controllerCompat.unregisterCallback(controllerCallback);
        }
        if (mMediaBrowser != null && mMediaBrowser.isConnected()) {
            mMediaBrowser.disconnect();
            mMediaBrowser = null;
        }
        MusicContainer.getInstance().removeBrowserHelper(this);
    }

    public boolean isPlayer() {
        return MusicContainer.getInstance().getMediaPlayer().isPlaying();
    }

    public MediaControllerCompat.TransportControls getTransportControls() {
        if (controllerCompat == null)
            return null;
        return controllerCompat.getTransportControls();
    }

    public void updateMusicData() {
        AppData.handler.post(new Runnable() {
            @Override
            public void run() {
                for (MediaControllerCompat.Callback callback : callbacks) {
                    if (callback != null) {
                        MusicContainer.getInstance().getNowPlayMetadataCompat(new AbsObjectInterface<MediaMetadataCompat>() {
                            @Override
                            public void onData(MediaMetadataCompat mediaMetadataCompat) {
                                callback.onMetadataChanged(mediaMetadataCompat);
                            }
                        });

                    }

                }

            }
        });
    }

    private final List<MediaControllerCompat.Callback> callbacks = new ArrayList<>();

    public void regPlayListener(MediaControllerCompat.Callback playListener) {
        callbacks.add(playListener);
    }

    public void unPlayListener(MediaControllerCompat.Callback playListener) {
        callbacks.remove(playListener);
    }

    public void play(MusicData item) {
        MusicContainer.getInstance().setNowPlayData(item);
        getTransportControls().play();
    }

    private class MediaBrowserConnectionCallback extends MediaBrowserCompat.ConnectionCallback {
        @Override
        public void onConnected() {
            super.onConnected();
            System.out.println("初始化:onConnected");
            controllerCompat = new MediaControllerCompat(context, mMediaBrowser.getSessionToken());
            controllerCompat.registerCallback(controllerCallback);
            controllerCallback.onMetadataChanged(controllerCompat.getMetadata());
            controllerCallback.onPlaybackStateChanged(controllerCompat.getPlaybackState());
            mMediaBrowser.subscribe(mMediaBrowser.getRoot(), subscriptionCallback);

        }
    }


    private class MediaControllerCallback extends MediaControllerCompat.Callback {
        @Override
        public void onPlaybackStateChanged(PlaybackStateCompat state) {
            super.onPlaybackStateChanged(state);
            for (MediaControllerCompat.Callback callback : callbacks) {
                callback.onPlaybackStateChanged(state);
            }
        }

        @Override
        public void onMetadataChanged(MediaMetadataCompat metadata) {
            super.onMetadataChanged(metadata);
            for (MediaControllerCompat.Callback callback : callbacks) {
                callback.onMetadataChanged(metadata);
            }
        }

        @Override
        public void onSessionDestroyed() {
            super.onSessionDestroyed();
            for (MediaControllerCompat.Callback callback : callbacks) {
                callback.onSessionDestroyed();
            }
        }

        @Override
        public void onQueueChanged(List<MediaSessionCompat.QueueItem> queue) {
            super.onQueueChanged(queue);
            for (MediaControllerCompat.Callback callback : callbacks) {
                callback.onQueueChanged(queue);
            }
        }
    }

    public static class MediaBrowserSubscriptionCallback extends MediaBrowserCompat.SubscriptionCallback {

    }
}
