package com.yutou.nas_music_player.containers;

import static android.media.MediaPlayer.SEEK_PREVIOUS_SYNC;

import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.support.v4.media.MediaMetadataCompat;
import android.widget.EditText;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Datas.PreviousPlayerList;
import com.yutou.nas_music_player.Interfaces.AbsObjectInterface;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.network.HttpManager;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.AppTools;
import com.yutou.nas_music_player.tools.ConfigTools;
import com.yutou.nas_music_player.tools.ImageLoader;
import com.yutou.nas_music_player.tools.StringUtil;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;
import java.util.stream.Collectors;
import java.util.stream.Stream;


public class MusicContainer {
    public static final int PLAY_MODEL_RANDOM = 0;//随机
    public static final int PLAY_MODEL_ORDER = 1;//顺序
    public static final int PLAY_MODEL_ONE_LOOP = 2;//单曲循环
    public static final int PLAY_MODEL_LIST_RANDOM = 3;//列表随机
    private static MusicContainer container;
    private final MusicLibs libs;
    private final MediaPlayer mediaPlayer;
    private int playIndex = -1;
    private MusicData nowPlayData = null;
    private MusicLibsInitInterface initInterface;
    private List<MediaPlayer.OnCompletionListener> completionListener;
    private final List<MediaBrowserHelper> browserHelpers;
    private List<MusicData> playList;
    private final List<MusicData> tmpList;
    private final LinkedHashMap<String, List<MusicData>> albumMapList;
    private final LinkedHashMap<String, List<MusicData>> artistMapList;
    private final LinkedHashMap<String, List<MusicData>> collectionMapList;
    private int playModel = PLAY_MODEL_RANDOM;
    private final PreviousPlayerList previousPlayerList;

    private MusicContainer(MusicLibsInitInterface initInterface) {
        this.initInterface = initInterface;
        mediaPlayer = new MediaPlayer();
        playList = new ArrayList<>();
        tmpList = new ArrayList<>();
        albumMapList = new LinkedHashMap<>();
        artistMapList = new LinkedHashMap<>();
        collectionMapList = new LinkedHashMap<>();
        previousPlayerList = new PreviousPlayerList();
        browserHelpers = new ArrayList<>();
        playIndex = ConfigTools.getConfigPreferences().getInt("playIndex", -1);
        initMediaPlayer();
        libs = new MusicLibs();
    }

    public void setPlayModel(int playModel) {
        this.playModel = playModel;
        ConfigTools.getConfigPreferences().edit().putInt("playModel", playModel).apply();
    }

    private void initMediaPlayer() {
        completionListener = new ArrayList<>();
        mediaPlayer.setLooping(false);
        mediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
            @Override
            public void onCompletion(MediaPlayer mp) {

                System.out.println(nowPlayData.getFile());
                int tmp = ((int) (nowPlayData.getDurationAsDouble() * 1000) - mp.getCurrentPosition());
                if (tmp > 1000) {
                    System.out.println("发生跳曲行为");
                    mp.start();
                    seekTo(mp.getCurrentPosition());
                } else {
                    System.out.println("自动播放完成，进行下一曲");
                    playTimer.cancel();
                    playTimer = null;
                    for (MediaBrowserHelper browserHelper : browserHelpers) {
                        browserHelper.getTransportControls().skipToNext();
                    }
                }
            }
        });
        mediaPlayer.setOnErrorListener(new MediaPlayer.OnErrorListener() {
            @Override
            public boolean onError(MediaPlayer mp, int what, int extra) {
                System.out.println("播放错误: what=" + what + " extra=" + extra);
                return false;
            }
        });
        mediaPlayer.setOnPreparedListener(new MediaPlayer.OnPreparedListener() {
            @Override
            public void onPrepared(MediaPlayer mp) {
                System.out.println("调用了onPrepared");
            }
        });
    }

    public void addCompletionListener(MediaPlayer.OnCompletionListener listener) {
        completionListener.add(listener);
    }

    public MediaPlayer getMediaPlayer() {
        return mediaPlayer;
    }

    public MusicData getNowPlayData() {
        return nowPlayData;
    }

    public void setNowPlayData(MusicData musicData) {
        this.nowPlayData = musicData;
    }

    public void addBrowserHelper(MediaBrowserHelper browserHelper) {
        browserHelpers.add(browserHelper);
    }

    public void removeBrowserHelper(MediaBrowserHelper browserHelper) {
        browserHelpers.remove(browserHelper);
    }

    public static MusicContainer getInstance() {
        return getInstance(null);
    }

    public static MusicContainer getInstance(MusicLibsInitInterface initInterface) {
        if (container == null) {
            container = new MusicContainer(initInterface);
        } else {
            if (initInterface != null)
                initInterface.init();
        }
        return container;
    }

    /**
     * 更新媒体控制器的数据
     */
    private void updateMusicData() {
        for (MediaBrowserHelper browserHelper : browserHelpers) {
            if (browserHelper != null)
                browserHelper.updateMusicData();
        }
    }

    /**
     * 随机下一首歌
     *
     * @param play             是否直接播放
     * @param networkInterface 获取完成
     */
    private void playOfAllRandom(boolean play, NetworkInterface networkInterface) {
        System.out.println("随机下一首");
        HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_RANDOM, new JSONObject(), new NetworkInterface() {
            @Override
            public void httpGetData(Object data, int state) {
                JSONObject json = JSONObject.parseObject((String) data);
                if (json.getInteger("code") == 0) {
                    String baseData = json.getString("data");
                    try {
                        json.put("path", URLDecoder.decode(baseData, "UTF-8"));
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                        json.put("path", baseData);
                    }
                    HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_METADATA, json, new NetworkInterface() {
                        @Override
                        public void httpGetData(Object data, int state) {
                            JSONObject json = JSONObject.parseObject((String) data);
                            System.err.println(data);
                            if (json.getInteger("code") == 0) {
                                nowPlayData = JSONArray.parseObject(json.getString("data"), MusicData.class);
                                if (networkInterface != null) {
                                    networkInterface.httpGetData(nowPlayData, state);
                                }
                                if (play) {
                                    play(nowPlayData);
                                    updateMusicData();
                                }
                            }
                        }

                        @Override
                        public void httpError(Exception e) {
                            if (networkInterface != null)
                                networkInterface.httpError(e);
                        }
                    });
                }
            }

            @Override
            public void httpError(Exception e) {

            }
        });
    }

    /**
     * 默认直接播放下一首
     */
    public void playOfAllRandom() {
        playList.clear();
        playOfAllRandom(true, null);
    }

    public void play(MusicData data) {
        System.out.println("准备播放：" + data.getTitle());
        if (isPause) {
            mediaPlayer.start();
            isPause = false;
            return;
        }
        ConfigTools.getConfigPreferences().edit().putInt("playIndex", playIndex).apply();
        nowPlayData = data;
        String url = data.getPlayUrl();
        previousPlayerList.add(data);
        try {
            if (mediaPlayer.isPlaying()) {
                mediaPlayer.stop();
            }
            mediaPlayer.reset();
            mediaPlayer.setDataSource(MyApplication.application, Uri.parse(url));
            mediaPlayer.prepare();
            mediaPlayer.start();
            if (playTimer == null) {
                playTimer = new Timer();
                playTimer.schedule(new TimerTask() {
                    @Override
                    public void run() {
                        for (MediaPlayer.OnCompletionListener listener : completionListener) {
                            if (listener != null) {
                                listener.onCompletion(mediaPlayer);
                            }
                        }
                    }
                }, 0, 1000);
            }
            MusicData tmp = data.clone();
            ConfigTools.getPreferences().edit().putString(ConfigTools.previous_music, JSONObject.toJSONString(tmp)).apply();
            MediaNotificationManager manager = MediaNotificationManager.getInstance();
            if (manager != null) {
                // manager.getService().updateNotification();
            }
            tmp = null;
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void seekTo(long pos) {
        System.out.println("跳帧:" + pos);
        if (mediaPlayer.isPlaying()) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                mediaPlayer.seekTo(pos, SEEK_PREVIOUS_SYNC);
            } else {
                mediaPlayer.seekTo((int) pos);
            }
        }
    }

    private Timer playTimer;

    public void getNowPlayMetadataCompat(AbsObjectInterface<MediaMetadataCompat> objectInterface) {
        if (nowPlayData == null) {
            objectInterface.onData(null);
        } else {
            getMetadataCompat(nowPlayData, objectInterface);
        }
    }

    public Long getYear(String year) {
        try {
            return Long.parseLong(year);
        } catch (Exception e) {
            if (year.contains("-")) {
                try {
                    return Long.parseLong(year.split("-")[0]);
                } catch (Exception e1) {
                    return 8888L;
                }
            }
        }
        return 8888L;
    }

    private MediaMetadataCompat metadataCompat;
    private String metaDataFile = "";

    public void getMetadataCompat(MusicData data, AbsObjectInterface<MediaMetadataCompat> objectInterface) {
        if (data.getMd5().equals(metaDataFile)) {
            objectInterface.onData(metadataCompat);
            return;
        }
        metaDataFile = data.getMd5();
        ImageLoader.getInstance().getImageBitmap(data.getImageUrl(), -1, -1, new AbsObjectInterface<Bitmap>() {
            @Override
            public void onData(Bitmap bitmap) {
                MediaMetadataCompat.Builder builder = new MediaMetadataCompat.Builder();
                builder.putString(MediaMetadataCompat.METADATA_KEY_MEDIA_ID, data.getTitle());
                builder.putString(MediaMetadataCompat.METADATA_KEY_ARTIST, data.getArtist());
                builder.putString(MediaMetadataCompat.METADATA_KEY_TITLE, data.getTitle());
                builder.putString(MediaMetadataCompat.METADATA_KEY_ALBUM, data.getAlbum());
                builder.putString(MediaMetadataCompat.METADATA_KEY_COMPOSER, data.getComposer());
                builder.putLong(MediaMetadataCompat.METADATA_KEY_YEAR, getYear(data.getYear()));
                builder.putLong(MediaMetadataCompat.METADATA_KEY_TRACK_NUMBER, Long.parseLong(data.getTrack()));
                builder.putLong(MediaMetadataCompat.METADATA_KEY_DISC_NUMBER, Long.parseLong(data.getDisc_no()));
                builder.putLong(MediaMetadataCompat.METADATA_KEY_DURATION, ((int) (data.getDurationAsDouble() * 1000)));
                if (bitmap != null) {
                    builder.putBitmap(MediaMetadataCompat.METADATA_KEY_DISPLAY_ICON, bitmap);
                }
                builder.putString("md5", data.getMd5());
                metadataCompat = builder.build();
                objectInterface.onData(metadataCompat);
            }
        });
    }

    public static MusicLibs getLibs() {
        return getInstance().libs;
    }

    private boolean isPause = false;

    public void pause() {
        isPause = true;
        mediaPlayer.pause();
    }

    public void play() {
        isPause = false;
        mediaPlayer.start();
    }

    public void receiverPause() {
        for (MediaBrowserHelper helper : browserHelpers) {
            if (helper.getTransportControls() != null) {
                helper.getTransportControls().pause();
            }
        }
    }

    /**
     * 播放下一曲
     */
    public void playNext() {
        System.out.println("播放模式:" + playModel + " index = " + playIndex + " play size = " + playList.size());
        if (tmpList.size() != 0) {//有插队列表，优先播放
            play(tmpList.get(0));//先插入的先播
            tmpList.remove(0);
            return;
        }
        if (playIndex < 0 || playIndex > playList.size()) {
            playIndex = 0;
        }
        switch (playModel) {
            case PLAY_MODEL_ORDER:
                playOrder(false);
                break;
            case PLAY_MODEL_ONE_LOOP:
                playOneLoop();
                break;
            case PLAY_MODEL_LIST_RANDOM:
                playOrder(true);
                break;
            case PLAY_MODEL_RANDOM:
            default:
                playOfAllRandom();
        }
    }

    /**
     * 播放上一曲
     */
    public void playPrevious() {
        playIndex--;
        if (playIndex < 0) {
            playIndex = 0;
        }
        MusicData data = previousPlayerList.getMusicData();
        if (data != null) {
            play(data);
        } else {
            Toast.makeText(MyApplication.application, "没有上一曲记录", Toast.LENGTH_LONG).show();
        }
    }

    /**
     * 顺序播放
     *
     * @param isRandom 随机播放
     */
    private void playOrder(boolean isRandom) {
        if (playList.isEmpty()) {//播放列表为空，以所有歌列表顺序播放
            playOfAllRandom();
            AppTools.toast("当前播放列表为空，随机播放");
        } else {//播放列表中的歌单
            if (isRandom) {
                playIndex = randomPlayIndex();
            } else {
                if (playList.size() > (playIndex + 1)) {
                    playIndex++;
                } else {
                    playIndex = 0;
                }
            }
            play(playList.get(playIndex));

        }
    }

    private int randomPlayIndex() {
        int tmp = new Random().nextInt(playList.size());
        if (tmp == playIndex) {
            return randomPlayIndex();
        }
        return tmp;
    }

    /**
     * 单曲循环
     */
    private void playOneLoop() {
        play(nowPlayData);
    }

    /**
     * 专辑播放
     *
     * @param isNext 下一首
     */
    private void playAlbumList(boolean isNext) {

    }

    public List<MusicData> getPlayList() {
        System.out.println("playList.size() = " + playList.size() + " tmpList.size() = " + tmpList.size());
        List<MusicData> tmp = new ArrayList<>();
        tmp.addAll(tmpList);
        tmp.addAll(playList);
        return tmp;
    }


    public void addTmpList(MusicData data) {
        if (data.getTitle().equals("返回") && data.getTrack().equals("-1") || data.isDir()) {
            return;
        }
        tmpList.add(data);
    }

    public void removeTmpList(MusicData data) {
        tmpList.remove(data);
    }

    public List<MusicData> getTmpPlayList() {
        return tmpList;
    }

    public LinkedHashMap<String, List<MusicData>> getAllAlbumMapList() {
        return albumMapList;
    }

    public LinkedHashMap<String, List<MusicData>> getAllArtistMapList() {
        return artistMapList;
    }

    public Map<String, List<MusicData>> getCollectionMap() {
        return collectionMapList;
    }

    public void setPlayList(List<MusicData> dataList) {
        this.playList = dataList;
        this.playIndex = -1;
        this.playModel = PLAY_MODEL_ORDER;
    }

    public void addPlayList(List<MusicData> dataList) {
        this.playList.addAll(dataList);
    }

    public void addPlayList(MusicData data) {
        this.playList.add(data);
    }

    public int getPlayModel() {
        return playModel;
    }

    public int getPlayIndex() {
        return playIndex;
    }

    public void setPlayIndex(int i) {
        this.playIndex = i;
    }

    public void clearPlayList() {
        playList.clear();
        tmpList.clear();
    }

    public class MusicLibs {

        private MusicLibs() {
            initData();
        }

        public void onAddCollection(String md5, int fid) {

        }

        public void onAddCollectionDir(Context context, MusicData... collectionData) {
            onCollectionList(null, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    List<MusicData> list = (List<MusicData>) data;
                    List<String> titles = new ArrayList<>();
                    titles.add("添加新收藏夹");
                    for (MusicData musicData : list) {
                        titles.add(musicData.getTitle());
                    }

                    AlertDialog dialog = new AlertDialog.Builder(context)
                            .setTitle("添加到收藏夹")
                            .setItems(titles.toArray(new CharSequence[list.size()]), new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    if (which == 0) {//添加新收藏夹
                                        EditText editText = new EditText(context);
                                        new AlertDialog.Builder(context)
                                                .setTitle("收藏夹名字")
                                                .setView(editText)
                                                .setPositiveButton("提交", new DialogInterface.OnClickListener() {
                                                    @Override
                                                    public void onClick(DialogInterface dialog, int which) {
                                                        if (StringUtil.isEmpty(editText.getText().toString())) {
                                                            Toast.makeText(context, "名字不能为空", Toast.LENGTH_LONG).show();
                                                            return;
                                                        }
                                                        JSONObject json = new JSONObject();
                                                        json.put("favorite", editText.getText().toString());
                                                        HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_Collection_Dir_Add, json, new NetworkInterface() {
                                                            @Override
                                                            public void httpGetData(Object data, int state) {
                                                                JSONObject json = JSONObject.parseObject((String) data);
                                                                Toast.makeText(context, json.getString("msg"), Toast.LENGTH_LONG).show();
                                                                dialog.dismiss();
                                                            }

                                                            @Override
                                                            public void httpError(Exception e) {
                                                                dialog.dismiss();
                                                            }
                                                        });
                                                    }
                                                })
                                                .setNegativeButton("取消", (dialog1, which1) -> dialog1.dismiss()).create().show();
                                    } else {//选择现有收藏夹
                                        MusicData musicData = list.get(which - 1);
                                        JSONArray array = new JSONArray();
                                        for (MusicData data : collectionData) {
                                            if (!StringUtil.isEmpty(data.getMd5()))
                                                array.add(data.getMd5());
                                        }
                                        JSONObject json = new JSONObject();
                                        json.put("fid", musicData.getId());
                                        json.put("md5s", array);
                                        HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_Collection_Add, json, new NetworkInterface() {
                                            @Override
                                            public void httpGetData(Object data, int state) {
                                                JSONObject json = JSONObject.parseObject((String) data);
                                                Toast.makeText(context, json.getString("msg"), Toast.LENGTH_LONG).show();
                                                dialog.dismiss();
                                            }

                                            @Override
                                            public void httpError(Exception e) {
                                                dialog.dismiss();
                                            }
                                        });

                                    }
                                }
                            })
                            .setNegativeButton("取消", new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    dialog.dismiss();
                                }
                            }).create();
                    dialog.show();

                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }

        public void onCollectionList(String collection, NetworkInterface networkInterface) {
            JSONObject json = new JSONObject();
            String url = HttpManager.NetworkAPI.MUSIC_Collection_Dir_List;
            if (collection != null) {
                url = HttpManager.NetworkAPI.MUSIC_Collection_Get;
                json.put("fid", Integer.parseInt(collection));
            }
            HttpManager.getInstance().httpPost(url, json, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    if (StringUtil.isEmpty((String) data)) {
                        networkInterface.httpGetData(new ArrayList<>(), 0);
                        return;
                    }
                    JSONObject json = JSONObject.parseObject((String) data);
                    if (json.getInteger("code") == 0) {
                        if (json.getJSONArray("data").toJSONString().contains("md5")) {
                            networkInterface.httpGetData(JSONArray.parseArray(json.getJSONArray("data").toJSONString(), MusicData.class), 0);
                        } else {
                            List<MusicData> list = new ArrayList<>();
                            for (Object album : json.getJSONArray("data")) {
                                JSONObject js = (JSONObject) album;
                                MusicData musicData = new MusicData();
                                musicData.setId(js.getInteger("id"));
                                musicData.setComment(js.getLong("count") + "");
                                musicData.setTitle(js.getString("title"));
                                musicData.setAlbum(js.getString("title"));
                                musicData.setDir(false);
                                list.add(musicData);
                            }
                            new Handler(Looper.getMainLooper())
                                    .post(new Runnable() {
                                        @Override
                                        public void run() {
                                            networkInterface.httpGetData(list, 0);

                                        }
                                    });
                        }
                    } else {
                        networkInterface.httpGetData(new ArrayList<>(), 0);
                    }
                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }

        public void removeCollection(int fid, NetworkInterface networkInterface, MusicData... musicData) {
            JSONObject json = new JSONObject();
            JSONArray array = new JSONArray();
            for (MusicData data : musicData) {
                JSONObject js = new JSONObject();
                js.put("md5", data.getMd5());
                js.put("fid", fid);
                array.add(js);
            }
            json.put("data", array);

            HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_Collection_Remove, json, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    JSONObject json = JSONObject.parseObject((String) data);
                    if (networkInterface != null) {
                        networkInterface.httpGetData(data, state);
                    }
                    Toast.makeText(MyApplication.application, json.getString("msg"), Toast.LENGTH_LONG).show();
                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }


        private void initData() {
            if (nowPlayData == null) {
                playOfAllRandom(false, new NetworkInterface() {
                    @Override
                    public void httpGetData(Object data, int state) {
                        //获取到下一首的信息，完成初始化
                        if (initInterface != null)
                            initInterface.init();
                        initInterface = null;
                    }

                    @Override
                    public void httpError(Exception e) {

                    }
                });
            } else {
                if (initInterface != null)
                    initInterface.init();
                initInterface = null;
            }
        }

        public void onAlbumMusicData(String album, NetworkInterface networkInterface) {
            JSONObject json = new JSONObject();
            if (StringUtil.isEmpty(album)) {
                json.put("album", "");
            } else {
                json.put("album", album);
            }
            HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_Album, json, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    JSONObject json = JSONObject.parseObject((String) data);
                    if (json.getInteger("code") == 0) {
                        try {
                            networkInterface.httpGetData(JSONArray.parseArray(json.getJSONArray("data").toJSONString(), MusicData.class), 0);
                        } catch (Exception e) {
                            List<MusicData> list = new ArrayList<>();
                            for (Object album : json.getJSONArray("data")) {
                                JSONObject _album = (JSONObject) album;
                                MusicData musicData = new MusicData();
                                musicData.setTitle(_album.getString("title"));
                                musicData.setAlbum(_album.getString("album"));
                                musicData.setMd5(_album.getString("md5"));
                                list.add(musicData);
                            }
                            new Handler(Looper.getMainLooper())
                                    .post(new Runnable() {
                                        @Override
                                        public void run() {
                                            networkInterface.httpGetData(list, 0);

                                        }
                                    });
                        }
                    } else {
                        networkInterface.httpGetData(new ArrayList<>(), 0);
                    }
                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }

        public void onDirMusicData(JSONObject json, NetworkInterface networkInterface) {
            HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_LIST, json, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    JSONObject json = JSONObject.parseObject((String) data);
                    if (json.getInteger("code") == 0) {
                        networkInterface.httpGetData(JSONArray.parseArray(json.getString("data"), MusicData.class), 0);
                    } else {
                        networkInterface.httpGetData(new ArrayList<>(), 0);
                    }
                }

                @Override
                public void httpError(Exception e) {
                    networkInterface.httpGetData(new ArrayList<>(), 0);
                }
            });
        }

        public void onDirMusicData(String path, NetworkInterface networkInterface) {
            if (StringUtil.isEmpty(path))
                path = "root";
            JSONObject json = new JSONObject();
            json.put("path", path);
            onDirMusicData(json, networkInterface);
        }

        public void getMusicList(String path, boolean type, NetworkInterface networkInterface) {
            JSONObject json = new JSONObject();
            json.put("path", path);
            json.put("type", type);
            HttpManager.getInstance().httpPost(HttpManager.NetworkAPI.MUSIC_LIST, json, new NetworkInterface() {
                @Override
                public void httpGetData(Object data, int state) {
                    JSONObject json = JSONObject.parseObject((String) data);
                    if (json.getInteger("code") == 0) {
                        networkInterface.httpGetData(JSONArray.parseArray(json.getJSONArray("data").toJSONString(), MusicData.class), 0);
                    }
                }

                @Override
                public void httpError(Exception e) {

                }
            });
        }

    }

    public interface MusicLibsInitInterface {
        void init();
    }
}
