package com.yutou.nas_music_player.network;


import android.os.Handler;
import android.os.Looper;

import androidx.annotation.NonNull;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.yutou.nas_music_player.Interfaces.DownloadInterface;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.AppTools;
import com.yutou.nas_music_player.tools.Log;
import com.yutou.nas_music_player.tools.NetworkTool;
import com.yutou.nas_music_player.tools.StringUtil;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class HttpManager {
    private static HttpManager manager;
    public static String downloadPath = AppData.dir + AppData.download_dir + File.separator;

    public static class NetworkAPI {
        public static final String HTTP_KEY = "OHreoKn2CSlDipWV6phEupAqPIX2LJFl";
        public static String HOME = "http://nas.cnmglz.com:8000/nas";
        public static String MUSIC_LIST = HOME + "/music/list.do";
        public static String MUSIC_Album = HOME + "/music/getAlbum.do";
        public static String MUSIC_Collection_Dir_List = HOME + "/music/favorite/dir/list.do";
        public static String MUSIC_Collection_Dir_Add = HOME + "/music/favorite/dir/add.do";
        public static String MUSIC_Collection_Dir_ReName = HOME + "/music/favorite/dir/rename.do";
        public static String MUSIC_Collection_Get = HOME + "/music/favorite/get.do";
        public static String MUSIC_Collection_Add = HOME + "/music/favorite/add.do";
        public static String MUSIC_Collection_Remove = HOME + "/music/favorite/remove.do";
        public static String MUSIC_ALL = HOME + "/music/all.do";
        public static String MUSIC_METADATA = HOME + "/music/find/file.do";
        public static String MUSIC_IMAGE = HOME + "/music/web/image.do";
        public static String MUSIC_PLAY = HOME + "/music/play.do";
        public static String MUSIC_RANDOM = HOME + "/music/random.do";


    }

    private HttpManager() {
        System.out.println("初始化NetWork");
        /*httpGet("http://192.168.31.88:8000/public/version.do", new NetworkInterface() {
            @Override
            public void httpGetData(Object data, int state) {
                System.out.println("state = " + state);
                if (state == 200) {
                    NetworkAPI.HOME = "http://192.168.31.88:8000/nas";
                    AppTools.toast("检测到局域网在线,使用局域网模式");
                } else {
                    toPublicNetWork();
                }
            }

            @Override
            public void httpError(Exception e) {
                toPublicNetWork();
            }

            private void toPublicNetWork() {
                System.out.println("使用互联网模式");
                httpPost("http://tools.yutou233.cn/nas/music/getlocalhost.do?token=" + NetworkAPI.HTTP_KEY, new JSONObject(), new NetworkInterface() {
                    @Override
                    public void httpGetData(Object data, int state) {
                        try {
                            JSONObject json = JSONObject.parseObject(data.toString());
                            if (json.getInteger("code") == 0) {
                                NetworkAPI.HOME = json.getString("data") + "/nas";
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void httpError(Exception e) {

                    }
                });
            }
        });*/
    }

    public static HttpManager getInstance() {
        if (manager == null) {
            manager = new HttpManager();
        }
        return manager;
    }

    public void httpGet(String url, NetworkInterface networkInterface) {
        if (StringUtil.isEmpty(url) || !url.startsWith("http")) {
            new Timer()
                    .schedule(new TimerTask() {
                        @Override
                        public void run() {
                            if (!StringUtil.isEmpty(NetworkAPI.HOME)) {
                                httpGet(NetworkAPI.HOME + url, networkInterface);
                            } else {
                                httpGet(url, networkInterface);
                            }

                        }
                    }, 300);
            return;
        }
        OkHttpClient client = getClient();
        Request request = new Request.Builder()
                .url(url)
                .build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@NonNull Call call, @NonNull IOException e) {
                networkInterface.httpError(e);
            }

            @Override
            public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                networkInterface.httpGetData(response, response.code());
            }
        });
    }

    public void httpPost(String url, JSONObject body, NetworkInterface networkInterface) {
        System.out.println("url = " + StringUtil.isEmpty(url) + " starts = " + !url.startsWith("http") + " data = " + url);
        if (StringUtil.isEmpty(url) || !url.startsWith("http")) {
            new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
                @Override
                public void run() {
                    if (!StringUtil.isEmpty(NetworkAPI.HOME)) {
                        httpPost(NetworkAPI.HOME + url, body, networkInterface);
                    } else {
                        httpPost(url, body, networkInterface);
                    }
                }
            }, 300);
            return;
        }
        OkHttpClient client = getClient();
        Request request = new Request.Builder()
                .url(url)
                .post(RequestBody.create(body.toJSONString(), MediaType.parse("application/json; charset=utf-8")))
                .build();
        client.newCall(request)
                .enqueue(new Callback() {
                    @Override
                    public void onFailure(@NonNull Call call, @NonNull IOException e) {
                        networkInterface.httpError(e);
                    }

                    @Override
                    public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                        String body = response.body().string();
                        int code = response.code();
                        AppData.handler.post(() -> {
                            networkInterface.httpGetData(body, code);
                        });
                    }
                });
    }

    public void download(String url, JSONObject body, String saveName, String method, DownloadInterface downloadInterface) {
        OkHttpClient client = getClient();
        Request request;
        Request.Builder requestBuilder = new Request.Builder();
        requestBuilder.url(url);
        if (method.equals("post")) {
            requestBuilder.post(RequestBody.create(body.toJSONString(), MediaType.parse("application/json; charset=utf-8")));
        }
        request = requestBuilder.build();
        client.newCall(request)
                .enqueue(new Callback() {
                    @Override
                    public void onFailure(@NonNull Call call, @NonNull IOException e) {
                        downloadInterface.onError(e);
                    }

                    @Override
                    public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                        File jar = null;
                        try {
                            File savePath = new File(downloadPath + saveName + "_tmp.tmp");
                            if (!savePath.exists()) {
                                savePath.getParentFile().mkdirs();
                            }
                            Log.i("Download", "下载文件:" + url + " 保存文件:" + saveName);

                            InputStream inputStream = response.body().byteStream();
                            jar = new File(downloadPath + saveName + "_tmp.tmp");
                            jar.createNewFile();
                            Log.i(NetworkTool.class, "临时保存文件:" + jar.getAbsolutePath());
                            OutputStream outputStream = new FileOutputStream(jar);
                            byte[] bytes = new byte[1024];
                            int len;
                            while ((len = inputStream.read(bytes)) > 0) {
                                outputStream.write(bytes, 0, len);
                            }
                            outputStream.close();
                            inputStream.close();
                            File oldJar = new File(downloadPath + saveName);
                            if (oldJar.exists()) {
                                oldJar.delete();
                            }
                            jar.renameTo(oldJar);
                            Log.i(NetworkTool.class, "实际保存：" + oldJar.getAbsolutePath() + " " + oldJar.getName());
                            Log.toast(MyApplication.application, "下载完成，重启生效");

                            if (downloadInterface != null) {
                                downloadInterface.onDownloadOver(oldJar);
                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                            if (downloadInterface != null)
                                downloadInterface.onError(e);

                        }
                    }
                });
    }

    private final List<String> downloadImageList = new ArrayList<>();

    public synchronized void downloadImage(String url, JSONObject body, String saveName, String method, DownloadInterface downloadInterface) {
        if(saveName.contains("/")){
            downloadImage(url, body, saveName.replace("/","_"), method, downloadInterface);
            return;
        }
        if (!StringUtil.isEmpty(NetworkAPI.HOME) && !url.startsWith("http")) {
            downloadImage(NetworkAPI.HOME + url, body, saveName, method, downloadInterface);
            return;
        }
        if (downloadImageList.contains(url + "?" + body.toJSONString())) {
            return;
        } else {
            downloadImageList.add(url + "?" + body.toJSONString());
        }
        OkHttpClient client = getClient();
        Request request;
        Request.Builder requestBuilder = new Request.Builder();
        requestBuilder.url(url);
        if (method.equals("post")) {
            FormBody.Builder bodyBuilder = new FormBody.Builder();
            for (String key : body.keySet()) {
                bodyBuilder.add(key, body.getString(key));
            }
            requestBuilder.post(bodyBuilder.build());
        }
        request = requestBuilder.build();
        client.newCall(request)
                .enqueue(new Callback() {
                    @Override
                    public void onFailure(@NonNull Call call, @NonNull IOException e) {
                        downloadInterface.onError(e);
                    }

                    @Override
                    public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                        File jar = null;
                        try {
                            File savePath = new File(downloadPath + saveName + "_tmp.tmp");
                            if (!savePath.exists()) {
                                savePath.getParentFile().mkdirs();
                            }
                            Log.i("donwloadImage", "下载文件:" + url + "?" + body + " 保存文件:" + saveName);
                            Log.i("downloadImageCode", response.code() + "");
                            if (response.code() != 200) {
                                downloadInterface.onError(new Exception("http error = " + response.code()));
                                return;
                            }

                            InputStream inputStream = response.body().byteStream();

                            jar = new File(downloadPath + File.separator + "image_cache" + File.separator + saveName + "_tmp.tmp");
                            if (!new File(downloadPath + File.separator + "image_cache").exists()) {
                                new File(downloadPath + File.separator + "image_cache").mkdirs();
                            }
                            if (!jar.exists()) {
                                jar.createNewFile();
                            }
                            Log.i(NetworkTool.class, "临时保存文件:" + jar.getAbsolutePath());
                            OutputStream outputStream = new FileOutputStream(jar);
                            byte[] bytes = new byte[1024];
                            int len;
                            while ((len = inputStream.read(bytes)) > 0) {
                                outputStream.write(bytes, 0, len);
                            }
                            outputStream.close();
                            inputStream.close();
                            if(jar.length()==0){
                                System.out.println("jar.getAbsolutePath() = " + jar.getAbsolutePath());
                                System.out.println("jar.length() = " + jar.length());
                                downloadInterface.onError(new FileNotFoundException("文件大小为空"));
                                jar.delete();
                                return;
                            }
                            File oldJar = new File(downloadPath + saveName);
                    /*if (oldJar.exists()) {
                        oldJar.delete();
                    }*/
                            jar.renameTo(oldJar);
                            Log.i(NetworkTool.class, "实际保存：" + oldJar.getAbsolutePath() + " " + oldJar.getName());
                            Log.toast(MyApplication.application, "下载完成，重启生效");

                            if (downloadInterface != null) {
                                AppData.handler.post(() -> {
                                    downloadInterface.onDownloadOver(oldJar);
                                });
                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                            if (downloadInterface != null)
                                downloadInterface.onError(e);
                        }
                        downloadImageList.remove(url + "?" + body.toJSONString());
                    }
                });
    }

    private OkHttpClient getClient() {
        OkHttpClient client = new OkHttpClient.Builder()
                .connectTimeout(3, TimeUnit.SECONDS)
                .build();

        return client;
    }
}
