package com.yutou.nas_music_player.services;

import android.app.Notification;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.SystemClock;
import android.support.v4.media.MediaBrowserCompat;
import android.support.v4.media.MediaMetadataCompat;
import android.support.v4.media.session.MediaControllerCompat;
import android.support.v4.media.session.MediaSessionCompat;
import android.support.v4.media.session.PlaybackStateCompat;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.media.MediaBrowserServiceCompat;

import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.AbsObjectInterface;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MediaNotificationManager;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.views.MainActivity;

import java.util.ArrayList;
import java.util.List;

public class MusicService extends MediaBrowserServiceCompat {
    private final static String TAG = "MusicService";
    public MediaSessionCompat mediaSession;
    private PlaybackStateCompat.Builder stateBuilder;
    private MediaNotificationManager notificationManager;
    private int mState = PlaybackStateCompat.STATE_PLAYING;
    private String playMd5 = "";
    private MediaBrowserHelper helper;

    private static final long MEDIA_SESSION_ACTIONS = PlaybackStateCompat.ACTION_PLAY
            | PlaybackStateCompat.ACTION_PAUSE
            | PlaybackStateCompat.ACTION_PLAY_PAUSE
            | PlaybackStateCompat.ACTION_SKIP_TO_NEXT
            | PlaybackStateCompat.ACTION_SKIP_TO_PREVIOUS
            | PlaybackStateCompat.ACTION_STOP
            | PlaybackStateCompat.ACTION_SEEK_TO;

    MusicReceiver receiver = new MusicReceiver();
    IntentFilter intentFilter = new IntentFilter(AudioManager.ACTION_AUDIO_BECOMING_NOISY);

    @Override
    public void onCreate() {
        super.onCreate();
        notificationManager = new MediaNotificationManager(this);
        mediaSession = new MediaSessionCompat(getApplicationContext(), "MEDIA_LOG");
        mediaSession.setFlags(
                MediaSessionCompat.FLAG_HANDLES_MEDIA_BUTTONS |
                        MediaSessionCompat.FLAG_HANDLES_TRANSPORT_CONTROLS);

        stateBuilder = new PlaybackStateCompat.Builder()
                .setActions(MEDIA_SESSION_ACTIONS);
        mediaSession.setPlaybackState(stateBuilder.build());
        mediaSession.setCallback(new MediaSessionCallback());
        Intent intent = new Intent(this, MainActivity.class);
        PendingIntent pi = PendingIntent.getActivity(this, 233, intent, PendingIntent.FLAG_MUTABLE);
        mediaSession.setSessionActivity(pi);
        setSessionToken(mediaSession.getSessionToken());
        MusicContainer.getInstance().addCompletionListener(new PlayCompletionListener());
        helper = MediaBrowserHelper.getInstance();
        helper.regPlayListener(new MediaControllerCompat.Callback() {
            private String compat_md5 = "";
            private MediaMetadataCompat compat;

            @Override
            public void onMetadataChanged(MediaMetadataCompat metadata) {
                super.onMetadataChanged(metadata);
                if (metadata == null) {
                    return;
                }
                String md5 = metadata.getString("md5");
                if (compat_md5.equals(md5)) {
                    return;
                }
                compat_md5 = md5;
                Notification notification = notificationManager.getNotification(metadata, builderState(0).build(), getSessionToken());
                notificationManager.getManager()
                        .notify(MediaNotificationManager.NOTIFICATION_ID, notification);
            }

            @Override
            public void onPlaybackStateChanged(PlaybackStateCompat state) {
                super.onPlaybackStateChanged(state);
                if (mState == state.getState()) {
                    return;
                }
                mState = state.getState();
                MusicData data = MusicContainer.getInstance().getNowPlayData();
                MusicContainer.getInstance().getMetadataCompat(data, new AbsObjectInterface<MediaMetadataCompat>() {
                    @Override
                    public void onData(MediaMetadataCompat mediaMetadataCompat) {
                        Notification notification = notificationManager.getNotification(mediaMetadataCompat, builderState(state.getPosition()).build(), getSessionToken());
                        notificationManager.getManager()
                                .notify(MediaNotificationManager.NOTIFICATION_ID, notification);
                    }
                });

            }
        });
        System.out.println("注册receiver");
        registerReceiver(receiver, intentFilter);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        unregisterReceiver(receiver);
    }

    @Nullable
    @Override
    public BrowserRoot onGetRoot(@NonNull String clientPackageName, int clientUid, @Nullable Bundle rootHints) {
        return new BrowserRoot("media_root_id", null);
    }

    @Override
    public void onLoadChildren(@NonNull String parentId, @NonNull Result<List<MediaBrowserCompat.MediaItem>> result) {
        List<MediaBrowserCompat.MediaItem> mediaItems = new ArrayList<>();
        MusicContainer.getInstance().getNowPlayMetadataCompat(new AbsObjectInterface<MediaMetadataCompat>() {
            @Override
            public void onData(MediaMetadataCompat compat) {
                System.out.println(compat);
                if (compat != null) {
                    MediaBrowserCompat.MediaItem mediaItem = new MediaBrowserCompat.MediaItem(compat.getDescription(), MediaBrowserCompat.MediaItem.FLAG_PLAYABLE);
                    mediaItems.add(mediaItem);
                }
                result.sendResult(mediaItems);
            }
        });

    }

    public void seekToNotification(long pos) {
        MusicData data = MusicContainer.getInstance().getNowPlayData();
        if (!playMd5.equals(data.getMd5())) {
            playMd5 = data.getMd5();
            MusicContainer.getInstance().getNowPlayMetadataCompat(new AbsObjectInterface<MediaMetadataCompat>() {
                @Override
                public void onData(MediaMetadataCompat mediaMetadataCompat) {
                    mediaSession.setMetadata(mediaMetadataCompat);
                    updateNotification();
                    mediaSession.setPlaybackState(new PlaybackStateCompat.Builder()
                            .setActions(MEDIA_SESSION_ACTIONS)
                            .setState(mState, pos, 1.0f, SystemClock.elapsedRealtime()).build());
                }
            });
        }
    }

    public PlaybackStateCompat.Builder builderState(long position) {
        PlaybackStateCompat.Builder stateBuilder = new PlaybackStateCompat.Builder();
        stateBuilder.setActions(getAvailableActions());
        stateBuilder.setState(mState,
                position,
                1.0f,
                SystemClock.elapsedRealtime());
        return stateBuilder;
    }

    private long getAvailableActions() {
        long actions = PlaybackStateCompat.ACTION_PLAY_FROM_MEDIA_ID
                | PlaybackStateCompat.ACTION_PLAY_FROM_SEARCH
                | PlaybackStateCompat.ACTION_SKIP_TO_NEXT
                | PlaybackStateCompat.ACTION_SKIP_TO_PREVIOUS;
        switch (mState) {
            case PlaybackStateCompat.STATE_STOPPED:
                actions |= PlaybackStateCompat.ACTION_PLAY
                        | PlaybackStateCompat.ACTION_PAUSE;
                break;
            case PlaybackStateCompat.STATE_PLAYING:
                actions |= PlaybackStateCompat.ACTION_STOP
                        | PlaybackStateCompat.ACTION_PAUSE
                        | PlaybackStateCompat.ACTION_SEEK_TO;
                break;
            case PlaybackStateCompat.STATE_PAUSED:
                actions |= PlaybackStateCompat.ACTION_PLAY
                        | PlaybackStateCompat.ACTION_STOP;
                break;
            default:
                actions |= PlaybackStateCompat.ACTION_PLAY
                        | PlaybackStateCompat.ACTION_PLAY_PAUSE
                        | PlaybackStateCompat.ACTION_STOP
                        | PlaybackStateCompat.ACTION_PAUSE;
        }
        return actions;
    }

    public void updateNotification() {

        MusicContainer.getInstance().getNowPlayMetadataCompat(new AbsObjectInterface<MediaMetadataCompat>() {
            @Override
            public void onData(MediaMetadataCompat metadataCompat) {
                System.out.println("更新通知栏:" + metadataCompat.getString(MediaMetadataCompat.METADATA_KEY_MEDIA_ID));
                //mediaSession.setMetadata(metadataCompat);
                mediaSession.setActive(true);
                Notification notification =
                        notificationManager.getNotification(
                                metadataCompat, builderState(0).build(), getSessionToken());
                notificationManager.getManager()
                        .notify(MediaNotificationManager.NOTIFICATION_ID, notification);
            }
        });

    }

    public class MediaSessionCallback extends MediaSessionCompat.Callback {

        boolean isRegReceiver = false;

        @Override
        public void onPlay() {
            super.onPlay();
            System.out.println("接收到播放指令");
            if (!isRegReceiver) {
                isRegReceiver = true;
            }
            mState = PlaybackStateCompat.STATE_PLAYING;
            MusicContainer.getInstance().getNowPlayMetadataCompat(new AbsObjectInterface<MediaMetadataCompat>() {
                @Override
                public void onData(MediaMetadataCompat metadataCompat) {
                    mediaSession.setMetadata(metadataCompat);
                    mediaSession.setActive(true);
                    Notification notification = notificationManager.getNotification(metadataCompat, builderState(0).build(), getSessionToken());
                    startForeground(MediaNotificationManager.NOTIFICATION_ID, notification);
                    MusicContainer.getInstance().play(MusicContainer.getInstance().getNowPlayData());
                }
            });

        }

        @Override
        public void onPause() {
            super.onPause();
            System.out.println("点击暂停");
            if (helper.isPlayer()) {
                mState = PlaybackStateCompat.STATE_PAUSED;
                MusicContainer.getInstance().pause();
            } else {
                mState = PlaybackStateCompat.STATE_PLAYING;
                MusicContainer.getInstance().play();
            }
            updateNotification();
            //MusicService.this.onSeekTo(-1);
        }


        @Override
        public void onSeekTo(long pos) {
            super.onSeekTo(pos);
            System.out.println("接收到跳帧指令~~~");
            MusicContainer.getInstance().seekTo(pos);
            MusicService.this.seekToNotification(pos);
        }

        @Override
        public void onSkipToNext() {
            super.onSkipToNext();
            System.out.println("播放下一曲");
            MusicContainer.getInstance().playNext();
            updateNotification();
        }

        @Override
        public void onSkipToPrevious() {
            super.onSkipToPrevious();
            MusicContainer.getInstance().playPrevious();
            updateNotification();
        }

        @Override
        public void onStop() {
            super.onStop();
            mState = PlaybackStateCompat.STATE_STOPPED;
            MusicContainer.getInstance().getMediaPlayer().stop();
            isRegReceiver = false;
        }
    }

    private class PlayCompletionListener implements MediaPlayer.OnCompletionListener {

        @Override
        public void onCompletion(MediaPlayer mp) {
            seekToNotification(mp.getCurrentPosition());
        }
    }
}
