package com.yutou.nas_music_player.tools;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Matrix;
import android.util.Base64;
import android.widget.Toast;

import androidx.palette.graphics.Palette;

import com.kaopiz.kprogresshud.KProgressHUD;
import com.yutou.nas_music_player.MyApplication;
import com.yutou.nas_music_player.R;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.security.MessageDigest;

public class AppTools {
    private static KProgressHUD hud;
    private static ImageLoader imageLoader;

    public static synchronized void showHud(Context context) {
        hideHud();
        hud = AppTools.showLoading(context);
        hud.show();
    }

    public static void hideHud() {
        if (hud != null && hud.isShowing()) {
            hud.dismiss();
            hud = null;
        }
    }

    public static KProgressHUD showLoading(Context context, String title) {
        return KProgressHUD.create(context)
                .setStyle(KProgressHUD.Style.SPIN_INDETERMINATE)
                .setDetailsLabel(title)
                .setCancellable(true)
                .setAnimationSpeed(2)
                .setDimAmount(0.5f)
                .show();
    }

    public static KProgressHUD showLoading(Context context) {
        return showLoading(context, "loading...");
    }

    public static void saveBitmap(final File img, final String saveKey) {
        ConfigTools.getPreferences().edit().putString("img_" + saveKey, img.getAbsolutePath()).apply();

    }

    public static Bitmap getSaveBitmap(String saveKey) {
        String path = ConfigTools.getPreferences().getString("img_" + saveKey, null);
        if (path == null) {
            return null;
        }
        File img = new File(path);
        if (img.length() > 3145728) {
            Bitmap bitmap = BitmapFactory.decodeFile(path);
            return compressQuality(bitmap);
        }
        return BitmapFactory.decodeFile(path);
    }

    private static Bitmap compressQuality(Bitmap bm) {
        Matrix matrix = new Matrix();
        matrix.setScale(0.5f, 0.5f);
        return Bitmap.createBitmap(bm, 0, 0, bm.getWidth(), bm.getHeight(), matrix, true);
    }

    public static String getMD5(String data) {
        try {
            MessageDigest digest = MessageDigest.getInstance("MD5");
            digest.update(data.getBytes());
            return bytesToHexString(digest.digest());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder();
        if (src == null || src.length <= 0) {
            return null;
        }
        for (byte aSrc : src) {
            int v = aSrc & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }


    public static String getUrlBase64(String text) {
        return new String(Base64.encode(text.getBytes(), Base64.DEFAULT)).replace("\n", "");
    }

    public static void toast(String s) {
        AppData.handler.post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(MyApplication.application, s, Toast.LENGTH_SHORT).show();
            }
        });
    }

    public static int getImageColor(Bitmap img) {
        try {
            return Palette.from(img).generate().getLightVibrantColor(Color.parseColor("#bbbbbb"));
        } catch (Exception e) {
            return Color.parseColor("#bbbbbb");
        }
    }
}
