package com.yutou.nas_music_player.tools;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.util.Log;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.Glide;
import com.bumptech.glide.RequestBuilder;
import com.bumptech.glide.request.FutureTarget;
import com.bumptech.glide.request.RequestOptions;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;
import com.yutou.nas_music_player.Interfaces.AbsObjectInterface;

import java.util.concurrent.ExecutionException;

import jp.wasabeef.glide.transformations.BlurTransformation;
import jp.wasabeef.glide.transformations.RoundedCornersTransformation;

public class ImageLoader {
    private static ImageLoader manager;
    private Context context;

    private ImageLoader(Context context) {
        this.context = context;
    }

    public static ImageLoader getInstance(Context context) {
        if (manager == null) {
            manager = new ImageLoader(context);
        }
        return manager;
    }

    public static ImageLoader getInstance() {
        return manager;
    }

    public void showImage(ImageView imageView, String url, int imageWidth, int imageHigh, boolean isRounded, boolean isBlurTransformation) {
        Log.w("图片地址", "url: " + url);
        RequestBuilder<Drawable> builder = Glide.with(context)
                .load(url)
                .placeholder(imageView.getDrawable());
        if (imageWidth != -1 && imageHigh != -1) {
            builder = builder.override(imageWidth, imageHigh);
        }
        if (isRounded) {
            builder = builder.apply(RequestOptions.bitmapTransform(new RoundedCornersTransformation(25, 3)));
        }
        if (isBlurTransformation) {
            builder = builder.apply(RequestOptions.bitmapTransform(new BlurTransformation(25, 3)));
        }
        builder.into(imageView);
    }

    public void showImage(ImageView imageView, String url) {
        showImage(imageView, url, -1, -1, false, false);
    }

    public void showImage(ImageView imageView, String url, int imageWidth, int imageHigh) {
        showImage(imageView, url, imageWidth, imageHigh, false, false);
    }

    public void showImage(ImageView imageView, String url, int imageWidth, int imageHigh, boolean isRounded) {
        showImage(imageView, url, imageWidth, imageHigh, isRounded, false);
    }

    public void getImageBitmap(String url, int imageWidth, int imageHigh, AbsObjectInterface<Bitmap> objectInterface) {
        RequestBuilder<Bitmap> builder = Glide.with(context)
                .asBitmap()
                .load(url);
        if (imageWidth != -1 && imageHigh != -1) {
            builder = builder.override(imageWidth, imageHigh);
        }
        builder.into(new CustomTarget<Bitmap>() {
            @Override
            public void onResourceReady(@NonNull Bitmap resource, @Nullable Transition<? super Bitmap> transition) {
                objectInterface.onData(resource);
            }

            @Override
            public void onLoadCleared(@Nullable Drawable placeholder) {

            }
        });

    }

}
