package com.yutou.nas_music_player.views;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.media.session.MediaControllerCompat;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.google.android.material.appbar.AppBarLayout;
import com.yutou.nas_music_player.Adapters.AlbumsRecyclerAdapter;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.R;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.ImageLoader;

import java.util.ArrayList;
import java.util.List;

import jp.wasabeef.glide.transformations.BlurTransformation;

public class AlbumsActivity extends AppCompatActivity {
    private ImageView album_image, image_background;
    private RecyclerView listView;
    private MediaBrowserHelper browserHelper;
    private AlbumsRecyclerAdapter adapter;
    private AppBarLayout appBarLayout;
    private TextView breakApp;
    private Button playAll;
    private int model;//0 专辑, 1 歌手, 2 收藏
    private PlayListener playListener;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        getWindow().requestFeature(Window.FEATURE_CONTENT_TRANSITIONS);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_album);
        model = getIntent().getIntExtra("model", -1);
        initView();
        switch (model) {
            case 2:
                initDataOfCollection(getIntent().getIntExtra("fid", -1));
                break;
            case 0:
                initData(getIntent().getStringExtra("album"));
                break;
            default:
                finish();
        }
        String imageUrl = getIntent().getStringExtra("image");
        if (imageUrl != null) {
            ImageLoader.getInstance(this).showImage(album_image, imageUrl, -1, -1, true);
            ImageLoader.getInstance(this).showImage(image_background, imageUrl, -1, -1, false,true);
        } else {
            album_image.setImageResource(R.mipmap.ic_launcher);
            Glide.with(AlbumsActivity.this).load(BitmapFactory.decodeResource(getResources(), R.mipmap.ic_launcher))
                    .apply(RequestOptions.bitmapTransform(new BlurTransformation(25, 3)))
                    .into(image_background);
        }
        playListener = new PlayListener();
        browserHelper = MediaBrowserHelper.getInstance();
        browserHelper.regPlayListener(playListener);
        adapter.setPlayListener(browserHelper);
        listView.setBackgroundColor(getIntent().getIntExtra("color", Color.parseColor("#bbbbbb")));
        playAll.setBackgroundColor(getIntent().getIntExtra("color", Color.parseColor("#bbbbbb")));

    }

    private static class PlayListener extends MediaControllerCompat.Callback {

    }

    private void initDataOfCollection(int fid) {
        adapter.setFid(fid);
        MusicContainer.getLibs().onCollectionList(fid + "", new NetworkInterface() {
            @Override
            public void httpGetData(Object data, int state) {
                List<MusicData> list = (List<MusicData>) data;
                adapter.setData(list);
                adapter.notifyDataSetChanged();
                if (list.isEmpty())
                    return;
                ImageLoader.getInstance(AlbumsActivity.this).showImage(image_background, list.get(0).getImageUrl(), AppData.imageWidth, AppData.imageHigh, true);
            }

            @Override
            public void httpError(Exception e) {

            }
        });
    }

    private void initData(String album) {
        MusicContainer.getLibs().onAlbumMusicData(album, new NetworkInterface() {
            @Override
            public void httpGetData(Object data, int state) {
                List<MusicData> list = (List<MusicData>) data;
                adapter.setData(list);
                AppData.handler.post(() -> {
                    adapter.notifyDataSetChanged();

                });
            }

            @Override
            public void httpError(Exception e) {

            }
        });

    }

    private void initView() {
        album_image = findViewById(R.id.album_image);
        image_background = findViewById(R.id.image_background);
        listView = findViewById(R.id.album_list);
        breakApp = findViewById(R.id.breakApp);
        appBarLayout = findViewById(R.id.appbar);
        playAll = findViewById(R.id.playAll);
        listView.setLayoutManager(new LinearLayoutManager(this));
        adapter = new AlbumsRecyclerAdapter(AlbumsActivity.this, new ArrayList<>(), model);
        listView.setAdapter(adapter);
        appBarLayout.addOnOffsetChangedListener((appBarLayout, verticalOffset) -> {
            if (verticalOffset < -200) {
                breakApp.setTextColor(getResources().getColor(R.color.musicArtist, null));
            } else {
                breakApp.setTextColor(getResources().getColor(R.color.musicTitle, null));
            }
        });
        breakApp.setOnClickListener(v -> finish());
        playAll.setOnClickListener(v -> {
            MusicContainer.getInstance().setPlayList(new ArrayList<>(adapter.getMusicDatas()));
            browserHelper.play(adapter.getMusicDatas().get(0));
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        adapter.onStop();
        browserHelper.unPlayListener(playListener);
    }
}
