package com.yutou.nas_music_player.views.Fragments;

import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_ALL;
import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_Album;
import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_Artist;
import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_PLAY_Collection;
import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_PLAY_LIST;
import static com.yutou.nas_music_player.Adapters.MusicLibsAdapter.LIBS_MODEL_Tmp;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityOptionsCompat;
import androidx.core.util.Pair;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.SimpleItemAnimator;

import com.yutou.nas_music_player.Adapters.MusicLibsAdapter;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.R;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.AppTools;

import java.util.ArrayList;
import java.util.List;

public class MusicLibsFragment extends Fragment {
    private Context context;
    private View view;
    private RecyclerView listView;
    private int model = 0;
    private MediaBrowserHelper browserHelper;

    public MusicLibsFragment() {

    }

    public MusicLibsFragment(Context context, int model) {
        this.context = context;
        this.model = model;
    }

    public void setPlayContainer(MediaBrowserHelper browserHelper) {
        this.browserHelper = browserHelper;
    }

    public void setModel(int model) {
        this.model = model;
        System.out.println("debug 设置model=" + model);
        adapter.initData(model);
        adapter.notifyDataSetChanged();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        browserHelper = null;
        adapter.onDestroy();
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        if (view != null)
            return view;
        view = inflater.inflate(R.layout.fragment_music_libs, container, false);
        if (adapter == null || listView == null)
            initViews();
        initData();
        return view;
    }

    private MusicLibsAdapter adapter;

    public void initViews() {
        listView = view.findViewById(R.id.listView);
        listView.setLayoutManager(new LinearLayoutManager(getContext()));
        adapter = new MusicLibsAdapter(getContext());
        listView.setAdapter(adapter);
        adapter.setBrowserHelper(browserHelper);
        adapter.setScrollStatus(1);
        adapter.setOnClickAction((intent, view) -> {
            Pair<View, String> pImage = Pair.create(view, "main_album_image");
            Bundle bundle = ActivityOptionsCompat.makeSceneTransitionAnimation(getActivity(), pImage).toBundle();
            getContext().startActivity(intent, bundle);
        });
        adapter.notifyDataSetChanged();
        closeDefaultAnimator();
    }
    /* * 关闭默认局部刷新动画 ，this代表recycleview*/
    public void closeDefaultAnimator() {
        listView.getItemAnimator().setAddDuration(0);
        listView.getItemAnimator().setChangeDuration(0);
        listView.getItemAnimator().setMoveDuration(0);
        listView.getItemAnimator().setRemoveDuration(0);
        ((SimpleItemAnimator) listView.getItemAnimator()).setSupportsChangeAnimations(false);
    }

    public MusicLibsAdapter getAdapter() {
        return adapter;
    }


    public void flash() {
        if (adapter == null)
            return;
        if (model == MusicLibsAdapter.LIBS_MODEL_Tmp) {
          //  adapter.initData(model);
            initData();
        }
        adapter.notifyDataSetChanged();
    }

    public void update() {
        if (adapter == null)
            return;
        adapter.initData(model);
        adapter.notifyDataSetChanged();
    }

    List<MusicData> list = new ArrayList<>();
    private final String musicPath = "root";

    public void initData() {
        AppTools.showHud(context);
        list.clear();
        adapter.setModel(model);
        switch (model) {
            case LIBS_MODEL_Album:
                String album = null;
                MusicContainer.getLibs().onAlbumMusicData(album, new NetworkInterface() {
                    @Override
                    public void httpGetData(Object data, int state) {
                        list.clear();
                        if (data != null && state == 0)
                            list.addAll((List<MusicData>) data);
                        adapter.setData(list);
                        adapter.notifyDataSetChanged();
                        AppTools.hideHud();
                    }

                    @Override
                    public void httpError(Exception e) {
                        AppTools.hideHud();
                    }
                });
                break;
            case LIBS_MODEL_Artist:
                AppTools.hideHud();
                break;
            case LIBS_MODEL_Tmp:
                list.addAll(MusicContainer.getInstance().getTmpPlayList());
                System.out.println("临时播放列表：" + MusicContainer.getInstance().getTmpPlayList().size());
                AppTools.hideHud();
                break;
            case LIBS_MODEL_PLAY_LIST:
                list.addAll(MusicContainer.getInstance().getPlayList());
                AppTools.hideHud();
                break;
            case LIBS_MODEL_PLAY_Collection:
                MusicContainer.getLibs().onCollectionList(null, new NetworkInterface() {

                    @Override
                    public void httpGetData(Object data, int state) {
                        list.clear();
                        if (data != null && state == 0)
                            list.addAll((List<MusicData>) data);
                        adapter.setData(list);
                        adapter.notifyDataSetChanged();
                        AppTools.hideHud();
                    }

                    @Override
                    public void httpError(Exception e) {
                        AppTools.hideHud();
                    }
                });
                break;
            case LIBS_MODEL_ALL:
            default:
                MusicContainer.getLibs().onDirMusicData(musicPath, new NetworkInterface() {
                    @Override
                    public void httpGetData(Object data, int state) {
                        list.clear();
                        if (data != null && state == 0)
                            list.addAll((List<MusicData>) data);
                        AppData.handler.post(new Runnable() {
                            @Override
                            public void run() {
                                adapter.setData(list);
                                AppTools.hideHud();
                                adapter.notifyDataSetChanged();
                            }
                        });
                    }

                    @Override
                    public void httpError(Exception e) {
                        AppData.handler.post(AppTools::hideHud);
                    }
                });

        }
        System.out.println("list.size() = " + list.size());
        adapter.setData(list);
        adapter.notifyDataSetChanged();
    }
}
