package com.yutou.nas_music_player.views;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.support.v4.media.MediaMetadataCompat;
import android.support.v4.media.session.MediaControllerCompat;
import android.support.v4.media.session.MediaSessionCompat;
import android.support.v4.media.session.PlaybackStateCompat;
import android.view.GestureDetector;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityOptionsCompat;
import androidx.core.util.Pair;
import androidx.core.view.GestureDetectorCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.alibaba.fastjson.JSON;
import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.yutou.nas_music_player.Adapters.MusicLibsAdapter;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.DefaultNetworkInterface;
import com.yutou.nas_music_player.R;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.AppTools;
import com.yutou.nas_music_player.tools.ConfigTools;
import com.yutou.nas_music_player.tools.ImageLoader;
import com.yutou.popdialog.POPDialog;

import java.util.List;
import java.util.Locale;
import java.util.Timer;
import java.util.TimerTask;

import cc.ibooker.ztextviewlib.MarqueeTextView;
import jp.wasabeef.glide.transformations.BlurTransformation;
import jp.wasabeef.glide.transformations.RoundedCornersTransformation;

public class MainActivity extends AppCompatActivity {
    private ImageButton play, previous, next, playModel;
    private MediaBrowserHelper browserHelper;
    private ImageView album_image, background_image;
    private MarqueeTextView title, album, artist;
    private TextView positionTime, durationTime, bitRate, playIndex;
    private SeekBar seekBar;
    private Handler handler;
    private PlayListener playListener;

    private GestureDetectorCompat touch;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
        super.onCreate(savedInstanceState);
        setBarColor(android.R.color.transparent);
        setContentView(R.layout.activity_main);
        handler = new Handler(Looper.myLooper());
        initView();
        play.setOnClickListener(v -> handler.post(() -> {

            if (browserHelper.isPlayer()) {
                browserHelper.getTransportControls().pause();
            } else {
                browserHelper.getTransportControls().play();
            }
        }));
        next.setOnClickListener(v -> handler.post(() -> browserHelper.getTransportControls().skipToNext()));
        previous.setOnClickListener(v -> handler.post(() -> browserHelper.getTransportControls().skipToPrevious()));
        seekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {

            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {
                handler.post(() -> {
                    if (browserHelper.getTransportControls() != null) {
                        browserHelper.getTransportControls().seekTo(MainActivity.this.seekBar.getProgress());
                    }
                });

            }
        });
        playModel.setOnClickListener(v -> {
            int model = (int) playModel.getTag();
            if (model == MusicContainer.PLAY_MODEL_LIST_RANDOM) {
                setPlayModelButton(MusicContainer.PLAY_MODEL_RANDOM, true);
                return;
            }
            setPlayModelButton(model + 1, true);
        });
        playListener = new PlayListener();
        browserHelper = MediaBrowserHelper.getInstance();
        browserHelper.regPlayListener(playListener);

        initData();
    }

    private void initData() {
        String previousMusic = ConfigTools.getPreferences().getString(ConfigTools.previous_music, null);
        MusicData data;
        if (previousMusic != null) {
            data = JSON.parseObject(previousMusic, MusicData.class);
            MusicContainer.getInstance().setNowPlayData(data);
            setPlayData(data);
        } else {
            data = MusicContainer.getInstance().getNowPlayData();
            if (data != null) {
                setPlayData(data);
            } else {
                Toast.makeText(this, "播放列表获取失败", Toast.LENGTH_LONG).show();
            }
        }
    }

    @SuppressLint("SetTextI18n")
    private void setPlayData(final MusicData data) {
        System.out.println("设置歌曲");
        System.out.println(data);
        if (data == null)
            return;
        title.setText(data.getTitle());
        artist.setText(data.getArtist());
        album.setText(data.getAlbum());
        bitRate.setText(data.getBitRate() + "kbps / " + data.getSampleRate() + "hz");
        if (MusicContainer.getInstance().getPlayList().isEmpty()) {
            playIndex.setText("(∞)");
        } else {
            playIndex.setText("(" + (MusicContainer.getInstance().getPlayIndex() + 1) + "/" + MusicContainer.getInstance().getPlayList().size() + ")");
        }
        seekBar.setMax((int) (data.getDurationAsDouble() * 1000));
        int minutes = (int) (data.getDurationAsDouble() / 60);
        int seconds = (int) (data.getDurationAsDouble() % 60);
        durationTime.setText(String.format(Locale.CHINA, "%02d", minutes) + ":"
                + String.format(Locale.CHINA, "%02d", seconds));
        int model = ConfigTools.getConfigPreferences().getInt("playModel", MusicContainer.PLAY_MODEL_RANDOM);
        setPlayModelButton(model);
        setImage(data);

    }

    private void setPlayModelButton(int model) {
        setPlayModelButton(model, false);
    }

    private void setPlayModelButton(int model, boolean isButton) {
        playModel.setTag(model);
        MusicContainer.getInstance().setPlayModel(model);
        switch (model) {
            case MusicContainer.PLAY_MODEL_RANDOM:
                playModel.setImageResource(R.drawable.ic_play_random);
                if (isButton)
                    AppTools.toast("全列表随机模式");
                break;
            case MusicContainer.PLAY_MODEL_ORDER:
                playModel.setImageResource(R.drawable.ic_play_list);
                if (isButton)
                    AppTools.toast("列表顺序模式");
                break;
            case MusicContainer.PLAY_MODEL_ONE_LOOP:
                playModel.setImageResource(R.drawable.ic_play_one_loop);
                if (isButton)
                    AppTools.toast("单曲循环");
                break;
            case MusicContainer.PLAY_MODEL_LIST_RANDOM:
                playModel.setImageResource(R.drawable.ic_play_loop);
                if (isButton)
                    AppTools.toast("列表随机");
                break;
            default:
                setPlayModelButton(MusicContainer.PLAY_MODEL_RANDOM, isButton);
        }
    }

    private void setImage(MusicData data) {
        // setBarColor(data.getImageColor());
        ImageLoader.getInstance(this).showImage(album_image, data.getImageUrl(), AppData.imageWidth, AppData.imageHigh, true);
        ImageLoader.getInstance(this).showImage(background_image, data.getImageUrl(),AppData.imageWidth, AppData.imageHigh, false, true);
    }

    private void initView() {
        play = findViewById(R.id.collection);
        next = findViewById(R.id.top);
        previous = findViewById(R.id.dislike);
        album_image = findViewById(R.id.album_image);
        background_image = findViewById(R.id.background_image);
        title = findViewById(R.id.title);
        artist = findViewById(R.id.artist);
        album = findViewById(R.id.album);
        bitRate = findViewById(R.id.bitRate);
        positionTime = findViewById(R.id.positionTime);
        durationTime = findViewById(R.id.durationTime);
        seekBar = findViewById(R.id.seekBar);
        playModel = findViewById(R.id.playModel);
        playIndex = findViewById(R.id.playIndex);
        touch = new GestureDetectorCompat(this, new MyTouch());
        int model = ConfigTools.getConfigPreferences().getInt("playModel", MusicContainer.PLAY_MODEL_RANDOM);
        setPlayModelButton(model);
    }

    public void setBarColor(int color) {
        //getWindow().setStatusBarColor(color);
        //  getWindow().setNavigationBarColor(Color.parseColor("#00FFFFFF"));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (browserHelper != null) {
            browserHelper.unPlayListener(playListener);
            browserHelper.onStop();
            browserHelper = null;
        }
        finish();
    }

    @Override
    protected void onResume() {
        super.onResume();
        setPlayData(MusicContainer.getInstance().getNowPlayData());
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);

    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            moveTaskToBack(true);
            return false;
        }
        return super.onKeyUp(keyCode, event);
    }


    @Override
    public boolean onTouchEvent(MotionEvent event) {
        return touch.onTouchEvent(event);
    }

    private void toPlayList() {
        Pair<View, String> pImage = Pair.create(album_image, "main_album_image");
        Pair<View, String> pPrevious = Pair.create(previous, "previous");
        Pair<View, String> pPlay = Pair.create(play, "play");
        Pair<View, String> pNext = Pair.create(next, "next");
        Pair<View, String> pTitle = Pair.create(title, "title");
        Pair<View, String> pArtist = Pair.create(artist, "artist");
        Pair<View, String> pBar = Pair.create(seekBar, "bar");
        Bundle bundle = ActivityOptionsCompat.makeSceneTransitionAnimation(MainActivity.this,
                pImage,
                pPrevious,
                pPlay,
                pNext,
                pBar,
                pTitle,
                pArtist).toBundle();

        Intent intent = new Intent(
                MainActivity.this, PlayLibsActivity.class);
        intent.putExtra("bar_pos", seekBar.getProgress());
        if (bundle != null) {
            startActivity(intent, bundle);
        } else {
            startActivity(intent);
        }
    }

    private void showPOPDialog() {
        List<MusicData> playList = MusicContainer.getInstance().getPlayList();
        POPDialog dialog = new POPDialog(this);
        RecyclerView recyclerView = null;
        Button clear = new Button(this);
        LinearLayout layout = new LinearLayout(this);
        layout.setOrientation(LinearLayout.VERTICAL);
        clear.setText("清空列表");
        dialog.enableExitButton(true);
        if (playList.isEmpty()) {
            dialog.setTitle("播放列表");
            dialog.setMessage("播放列表为空,正在全列表随机播放");
        } else {
            dialog.setTitle("播放列表:" + playList.size());
            MusicLibsAdapter adapter = new MusicLibsAdapter(this);
            //    ItemTouchHelper touchHelper=new ItemTouchHelper(new MusicItemTouchAdapterCallback());
            ViewGroup.LayoutParams params = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
            recyclerView = new RecyclerView(this);
            recyclerView.setLayoutManager(new LinearLayoutManager(this));
            recyclerView.setLayoutParams(params);
            recyclerView.setAdapter(adapter);
            //      touchHelper.attachToRecyclerView(recyclerView);
            adapter.setData(playList);
            adapter.setBrowserHelper(browserHelper);
            clear.setOnClickListener(view -> {
                MusicContainer.getInstance().clearPlayList();
                AppTools.toast("列表已清空");
                dialog.dismiss();
            });
            layout.addView(clear);
            layout.addView(recyclerView);
            dialog.setContentView(layout);
        }
        dialog.show();
    }

    private class PlayListener extends MediaControllerCompat.Callback {
        @SuppressLint("SetTextI18n")
        @Override
        public void onPlaybackStateChanged(PlaybackStateCompat state) {
            super.onPlaybackStateChanged(state);
            //System.out.println("[MainActivity]播放状态变化:" + state.getPosition() + " " + seekBar.getMax()+" state.getState() = "+state.getState()+" isPlay = "+browserHelper.isPlayer());
            if (state.getState() == PlaybackStateCompat.STATE_PLAYING || browserHelper.isPlayer()) {
                int duration = (int) (state.getPosition() / 1000);
                int minutes = duration / 60;
                int seconds = duration % 60;
                positionTime.setText(String.format(Locale.CHINA, "%02d", minutes) + ":"
                        + String.format(Locale.CHINA, "%02d", seconds));
                seekBar.setProgress((int) (state.getPosition()));

                play.setImageResource(android.R.drawable.ic_media_pause);
            } else {
                play.setImageResource(android.R.drawable.ic_media_play);
            }

        }

        @Override
        public void onMetadataChanged(MediaMetadataCompat metadata) {
            super.onMetadataChanged(metadata);
            String md5 = null;
            try {
                System.out.println("[MainActivity]播放歌曲变换:" + metadata.getString("md5"));
                md5 = metadata.getString("md5");
            } catch (Exception ignored) {
            }

            if (md5 != null) {
                setPlayData(MusicContainer.getInstance().getNowPlayData());
            }
        }

        @Override
        public void onSessionDestroyed() {
            super.onSessionDestroyed();
        }

        @Override
        public void onQueueChanged(List<MediaSessionCompat.QueueItem> queue) {
            super.onQueueChanged(queue);
        }
    }

    private class MyTouch extends GestureDetector.SimpleOnGestureListener {

        @Override
        public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {
            float x = e1.getX() - e2.getX();
            float y = e1.getY() - e2.getY();
            if (Math.abs(x) < 200 && y > 300) {
                showPOPDialog();
            } else if (Math.abs(x) < 200 && y < -500) {
                toPlayList();
            } else if (Math.abs(y) < 200 && x > 300) {
                next.callOnClick();
            } else if (Math.abs(y) < 200 && x < -300) {
                previous.callOnClick();
            }
            return super.onFling(e1, e2, velocityX, velocityY);
        }
    }
}