package com.yutou.nas_music_player.views;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.media.MediaMetadataCompat;
import android.support.v4.media.session.MediaControllerCompat;
import android.support.v4.media.session.MediaSessionCompat;
import android.support.v4.media.session.PlaybackStateCompat;
import android.view.View;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ProgressBar;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.viewpager2.adapter.FragmentStateAdapter;
import androidx.viewpager2.widget.ViewPager2;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;
import com.google.android.material.tabs.TabLayout;
import com.google.android.material.tabs.TabLayoutMediator;
import com.yutou.nas_music_player.Adapters.MusicLibsAdapter;
import com.yutou.nas_music_player.Datas.MusicData;
import com.yutou.nas_music_player.Interfaces.NetworkInterface;
import com.yutou.nas_music_player.R;
import com.yutou.nas_music_player.containers.MediaBrowserHelper;
import com.yutou.nas_music_player.containers.MusicContainer;
import com.yutou.nas_music_player.tools.AppData;
import com.yutou.nas_music_player.tools.ImageLoader;
import com.yutou.nas_music_player.views.Fragments.MusicLibsFragment;

import java.util.ArrayList;
import java.util.List;

import cc.ibooker.ztextviewlib.MarqueeTextView;
import jp.wasabeef.glide.transformations.RoundedCornersTransformation;

public class PlayLibsActivity extends AppCompatActivity {
    private ImageButton play, previous, next;
    private ProgressBar progressBar;
    private MediaBrowserHelper browserHelper;
    private ImageView album_image;
    private MarqueeTextView title, artist;
    private Handler handler;
    private int bar_pos = 0;

    private TabLayout tabLayout;
    private ViewPager2 viewPager;
    private final int index = 0;
    PlayListener playListener = new PlayListener();

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_play_list);
        browserHelper = MediaBrowserHelper.getInstance();
        browserHelper.regPlayListener(playListener);
        initView();
        bar_pos = getIntent().getIntExtra("bar_pos", 0);
        setPlayData(MusicContainer.getInstance().getNowPlayData());
        handler = new Handler();
        play.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        if (browserHelper.isPlayer()) {
                            browserHelper.getTransportControls().pause();
                        } else {
                            browserHelper.getTransportControls().play();
                        }

                    }
                });

            }
        });
        next.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        browserHelper.getTransportControls().skipToNext();
                    }
                });

            }
        });
        previous.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        browserHelper.getTransportControls().skipToPrevious();

                    }
                });
            }
        });
    }

    List<MusicLibsFragment> fragments;

    private void initView() {
        play = findViewById(R.id.collection);
        next = findViewById(R.id.top);
        previous = findViewById(R.id.dislike);
        album_image = findViewById(R.id.album_image);
        title = findViewById(R.id.title);
        artist = findViewById(R.id.artist);
        progressBar = findViewById(R.id.progressBar);
        tabLayout = findViewById(R.id.tabLayout);
        viewPager = findViewById(R.id.viewPager);
        fragments = new ArrayList<>();
        fragments.add(new MusicLibsFragment(this, MusicLibsAdapter.LIBS_MODEL_ALL));
        fragments.add(new MusicLibsFragment(this, MusicLibsAdapter.LIBS_MODEL_Album));
        fragments.add(new MusicLibsFragment(this, MusicLibsAdapter.LIBS_MODEL_PLAY_Collection));
        fragments.add(new MusicLibsFragment(this, MusicLibsAdapter.LIBS_MODEL_Tmp));
        for (MusicLibsFragment fragment : fragments) {
            fragment.setPlayContainer(browserHelper);
        }
        viewPager.setAdapter(new FragmentStateAdapter(this) {
            @NonNull
            @Override
            public Fragment createFragment(int position) {
                return fragments.get(position);
            }

            @Override
            public int getItemCount() {
                return fragments.size();
            }
        });
        tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                fragments.get(tab.getPosition()).flash();
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {
            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {
                fragments.get(tab.getPosition()).update();
            }
        });
        TabLayoutMediator mediator = new TabLayoutMediator(tabLayout, viewPager, true, new TabLayoutMediator.TabConfigurationStrategy() {
            @Override
            public void onConfigureTab(@NonNull TabLayout.Tab tab, int position) {
                switch (position) {
                    case 0:
                        tab.setText("全部");
                        break;
                    case 1:
                        tab.setText("专辑");
                        break;
                    case 2:
                        tab.setText("收藏");
                        break;
                    case 3:
                        tab.setText("队列");
                        break;
                }
            }
        });
        mediator.attach();


    }


    @SuppressLint("SetTextI18n")
    private void setPlayData(final MusicData data) {
        if (data == null)
            return;
        title.setText(data.getTitle());
        artist.setText(data.getArtist());
        progressBar.setMax((int) (data.getDurationAsDouble() * 1000));
        progressBar.setProgress(bar_pos);
        ImageLoader.getInstance(this).showImage(album_image, data.getImageUrl(), AppData.minImageWidth, AppData.minImageHigh, true);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        browserHelper.unPlayListener(playListener);
        browserHelper = null;
        finish();
    }


    private class PlayListener extends MediaControllerCompat.Callback {
        @SuppressLint("SetTextI18n")
        @Override
        public void onPlaybackStateChanged(PlaybackStateCompat state) {
            super.onPlaybackStateChanged(state);
            if (state.getState() == PlaybackStateCompat.STATE_PLAYING || browserHelper.isPlayer()) {
                progressBar.setProgress((int) (state.getPosition()));
                play.setImageResource(android.R.drawable.ic_media_pause);
            } else {
                play.setImageResource(android.R.drawable.ic_media_play);
            }

        }

        @Override
        public void onMetadataChanged(MediaMetadataCompat metadata) {
            super.onMetadataChanged(metadata);
            String md5 = null;
            try {
                System.out.println("播放歌曲变换:" + metadata.getString("md5"));
                md5 = metadata.getString("md5");
            } catch (Exception ignored) {
            }

            if (md5 != null) {
                setPlayData(MusicContainer.getInstance().getNowPlayData());
            }
        }

        @Override
        public void onSessionDestroyed() {
            super.onSessionDestroyed();
            System.out.println("连接被销毁");
        }

        @Override
        public void onQueueChanged(List<MediaSessionCompat.QueueItem> queue) {
            super.onQueueChanged(queue);
            System.out.println("不知道是啥变换了");
        }
    }
}
